// PIV: ANSI C implementation of the API in SP 800-73, Chapter 6 
//
// COPYRIGHT
// ---------
// The software was developed by the National Institute of Standards and 
// Technology (NIST), an agency of the Federal Government.  Pursuant to 
// Title 15 United States Code Section 105, works of NIST are not subject
// to copyright protection in the United States and are considered to be
// in the public domain.  As a result, a formal license is not needed to
// use the software.
// 
// The PIV Middleware Reference Implementation software is provided
// by NIST as a service and is expressly provided "AS IS."  
// NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.
// NIST does not warrant or make any representations regarding the use of
// the software or the results thereof, including but not limited to the
// correctness, accuracy, reliability or usefulness of the software.
// 
// Permission to use this software is contingent upon your acceptance of 
// the terms of this agreement and upon your providing appropriate
// acknowledgments of NIST's ownership of the software.
#ifndef __piv_h__
#define __piv_h__

#ifdef __cplusplus
extern "C" {
#endif

#ifndef __APPLE__
#include <winscard.h>
#else
#include <PCSC/wintypes.h>
#include <PCSC/winscard.h>
#endif



#include "pivdefs.h"


/* Function Codes */
#define PIV_CONNECT                            1
#define PIV_DISCONNECT                         2
#define PIV_SELECT_CARD_APPLICATION            3
#define PIV_LOG_IN_TO_CARD_APPLICATION         4
#define PIV_GET_DATA                           5
#define PIV_LOG_OUT_OF_CARD_APPLICATION        6
#define PIV_CRYPT                              7
#define PIV_PUT_DATA                           8
#define PIV_GENERATE_KEY_PAIR                  9
#define PIV_OC_HACK                           10

void DYNAMIC_LIB dumpData(char* memAddr,unsigned long len);

/* Entry points for Communication */
DYNAMIC_LIB PIV_RV pivConnect (PIV_Bool sharedConnection, 
				   PIV_Byte *connectionDescription, 
				   PIV_ULong32 *pCDLength, 
				   PIV_CARDHANDLE *pCardHandle);

DYNAMIC_LIB PIV_RV pivDisconnect (PIV_CARDHANDLE cardHandle);


/*  Entry Points for Data Access */
DYNAMIC_LIB PIV_RV pivLogIntoCardApplication (PIV_CARDHANDLE cardHandle, 
								  PIV_Byte *authenticators, 
								  PIV_ULong32 authLength);

DYNAMIC_LIB PIV_RV pivSelectCardApplication (PIV_CARDHANDLE cardHandle, 
								 PIV_Byte *applicationAID, 
								 PIV_ULong32 aidLength, 
								 PIV_Byte *applicationProperties, 
								 PIV_ULong32 *pAPLength);

DYNAMIC_LIB PIV_RV pivGetData (PIV_CARDHANDLE cardHandle, 
				   PIV_Byte *OID, 
				   PIV_ULong32 oidLength, 
				   PIV_Byte *data, 
				   PIV_ULong32 *pDataLength);

DYNAMIC_LIB PIV_RV pivLogoutOfCardApplication (PIV_CARDHANDLE cardHandle);


/*  Entry Points for Cryptographic Operations */
DYNAMIC_LIB PIV_RV pivCrypt (PIV_CARDHANDLE cardHandle, 
				 PIV_Byte algID, 
				 PIV_Byte keyReference, 
				 PIV_Byte *algInput, 
				 PIV_ULong32 inputLength, 
				 PIV_Byte *algOutput, 
				 PIV_ULong32 *pOutputLength);

/* Entry Points for Credential Initialization and Administration */
DYNAMIC_LIB PIV_RV pivPutData (PIV_CARDHANDLE cardHandle, 
				   PIV_Byte *OID, 
				   PIV_ULong32 oidLength, 
				   PIV_Byte *data, 
				   PIV_ULong32 dataLength);

DYNAMIC_LIB PIV_RV pivGenerateKeyPair (PIV_CARDHANDLE cardHandle, 
						   PIV_Byte keyReference, 
						   PIV_Byte cryptographicMechanism, 
						   PIV_Byte *publicKey, 
						   PIV_ULong32 *pKeyLength);

DYNAMIC_LIB PIV_RV StatusWord2RV (StatusWord sw);

#ifdef __cplusplus
}
#endif

#endif
