// APDU_Client - Common APDU Exchange Function for APDU Clients
//
// COPYRIGHT
// ---------
// The software was developed by the National Institute of Standards and 
// Technology (NIST), an agency of the Federal Government.  Pursuant to 
// Title 15 United States Code Section 105, works of NIST are not subject
// to copyright protection in the United States and are considered to be
// in the public domain.  As a result, a formal license is not needed to
// use the software.
// 
// The Government Smart Card Reference Implementation software is provided
// by NIST as a service and is expressly provided "AS IS."  
// NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.
// NIST does not warrant or make any representations regarding the use of
// the software or the results thereof, including but not limited to the
// correctness, accuracy, reliability or usefulness of the software.
// 
// Permission to use this software is contingent upon your acceptance of 
// the terms of this agreement and upon your providing appropriate
// acknowledgments of NIST's ownership of the software.

#include <APDU_Client.h>
//
// Utility function to exchange an APDU with logging in a standard format
//   - the cAPDU is sent from command
//   - the rAPDU is received into response
//   - the client is requested to place comment in its log file
//   - the exchange is logged 
//   - the client exchange result is returned
//
long APDU_Client::exchange_APDUs( const CAPDU_Packet & command, 
                                  RAPDU_Packet & response , 
                                  const char * comment)
{
  long result ;
  if ( NULL != comment ) 
    {
      result = the_APDU_server . log_file_comment ( comment ) ;
      if ( 0 != result )
        return result ;
    }

  log_message ( "SENT->%08X %04X" , command . get_APDU_length ( ) , command . get_logical_card_identifier ( ));
  for ( size_t i = 0 ; i < command  . get_APDU_length ( ) ; i ++ )
    log_message ( " %02X", command . get_APDU_octet ( i ) ) ;
  log_message ( "\n" ) ;

  result = the_APDU_server . exchange_APDUs( command , response ) ;

  if ( 0 != result )
    return result ;

  log_message ( "RECV<-%08X %04X"  , response . get_APDU_length ( ) , response . get_logical_card_identifier ( ));
  for ( size_t i = 0 ; i < response  . get_APDU_length ( ) ; i ++ )
    log_message ( " %02X", response . get_APDU_octet ( i ) ) ;
  log_message ( "\n" ) ;

  return 0 ;
}
