// Messages
//
// COPYRIGHT
// ---------
// The software was developed by the National Institute of Standards and 
// Technology (NIST), an agency of the Federal Government.  Pursuant to 
// Title 15 United States Code Section 105, works of NIST are not subject
// to copyright protection in the United States and are considered to be
// in the public domain.  As a result, a formal license is not needed to
// use the software.
// 
// The Government Smart Card Reference Implementation software is provided
// by NIST as a service and is expressly provided "AS IS."  
// NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.
// NIST does not warrant or make any representations regarding the use of
// the software or the results thereof, including but not limited to the
// correctness, accuracy, reliability or usefulness of the software.
// 
// Permission to use this software is contingent upon your acceptance of 
// the terms of this agreement and upon your providing appropriate
// acknowledgments of NIST's ownership of the software.

//--------------------------------------------------------------------------
// Debugging printout
//
// Use the debugger output console if availabe.
//
// NOTE: Extremely Win32-specific.
//
// Messages.c should NOT be included as part of any library , 
// but should be compiled along with the "main" ( "WinMain" , whatever ) 
// of the project being compiled , so that it can pick up the
// appropriate value of _CONSOLE.
//
//--------------------------------------------------------------------------

#include <Messages.h>
#include <stdio.h>
FILE * logfile;
#if defined(_WIN32 ) 
  #undef _WIN32_WINNT
  #define _WIN32_WINNT 0x501
  #include <windows.h>

  #define vsnprintf _vsnprintf
#endif
#include <macros.h>

#ifndef _CONSOLE
   int UseMessageBox = FALSE ;
#endif

#define BUFFERSIZE 4096

static char Buffer[BUFFERSIZE + 1]={0};

int debug_verbosity = 0 ; // set to non-zero to enable debug output

void log_message ( const char * format , ... ) 
{
  va_list vals;

  va_start ( vals , format ) ;     /* Initialize variable arguments. */

  vsnprintf ( Buffer , BUFFERSIZE , format , vals ) ;

#ifdef _CONSOLE
	if ( NULL != logfile )  
    fputs ( Buffer , logfile ) ;
#else
  if ( IsDebuggerPresent ( ) ) 
    OutputDebugString ( Buffer ) ;
#endif
}


void error_message ( const char * format , ... ) 
{
  va_list vals;

  va_start ( vals , format ) ;     /* Initialize variable arguments. */

  vsnprintf ( Buffer , BUFFERSIZE , format , vals ) ;

#ifdef _CONSOLE
	fputs ( Buffer , stderr ) ;
#else
  MessageBox ( NULL , Buffer , "PIV Card Application Reference Implementation" , MB_OK ) ;
#endif
}


void debug_message ( const char * format , ... ) 
{
  if ( debug_verbosity ) 
    {
#ifdef _DEBUG
      va_list vals;

      va_start ( vals , format ) ;     /* Initialize variable arguments. */

      vsnprintf ( Buffer , BUFFERSIZE , format , vals ) ;

      if ( IsDebuggerPresent ( ) ) 
        OutputDebugString ( Buffer ) ;
#ifdef _CONSOLE
      else 
        fputs ( Buffer , stdout ) ;
#else
      else if ( UseMessageBox ) 
	      MessageBox ( NULL , Buffer , "PIV Card Application Reference Implementation" , MB_OK ) ;
#endif   // _CONSOLE
#else
  UNREFERENCED_PARAMETER ( format ) ;
#endif   // _DEBUG
  }
}
