// Octet_String - ( start , stop ) substrings of Octet storage
//
// COPYRIGHT
// ---------
// The software was developed by the National Institute of Standards and 
// Technology (NIST), an agency of the Federal Government.  Pursuant to 
// Title 15 United States Code Section 105, works of NIST are not subject
// to copyright protection in the United States and are considered to be
// in the public domain.  As a result, a formal license is not needed to
// use the software.
// 
// The Government Smart Card Reference Implementation software is provided
// by NIST as a service and is expressly provided "AS IS."  
// NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.
// NIST does not warrant or make any representations regarding the use of
// the software or the results thereof, including but not limited to the
// correctness, accuracy, reliability or usefulness of the software.
// 
// Permission to use this software is contingent upon your acceptance of 
// the terms of this agreement and upon your providing appropriate
// acknowledgments of NIST's ownership of the software.

#include "Octet_String.h"
#include <strstream>


string
Octet_String::spell ( void ) const
{
  ostrstream spelling_stream;
  if ( the_start < the_stop )
  {
    
      for ( Memory::offset_t i = the_start ; i < the_stop ; i ++ )
      {
        spelling_stream << "0x" ;
        spelling_stream.unsetf( ios_base::dec );
        spelling_stream.setf( ios_base::hex );
        spelling_stream.width(2);
        spelling_stream.fill('0');
        spelling_stream << ( short ) the_storage -> get ( i ) ;
        if ( i < the_stop - 1 )
          spelling_stream << " " ;
      }
  }
  spelling_stream << ends ;

  return spelling_stream . str ( ) ;
}

string
Octet_String::spell_with_indentation ( string indentation ) const
{
  ostrstream spelling_stream;
  if ( the_start < the_stop )
  {
      spelling_stream << indentation . c_str ( ) ;
      for ( Memory::offset_t i = the_start ; i < the_stop ; i ++ )
      {
        spelling_stream << "0x" ;
        spelling_stream.unsetf( ios_base::dec );
        spelling_stream.setf( ios_base::hex );
        spelling_stream.width(2);
        spelling_stream.fill('0');
        spelling_stream << ( short ) the_storage -> get ( i ) ;
        if ( i < the_stop - 1 )
          {
            if ( ( i - the_start) % 8 == 7 )
              spelling_stream << endl << indentation . c_str ( )  ;
            else
              spelling_stream << " " ;
          }
      }
  }
  spelling_stream << ends ;

  return spelling_stream . str ( ) ;
}
int 
Octet_String::get_verbosity ( void ) { return verbosity ; } ;

int 
Octet_String::set_verbosity ( int v ) { return ( verbosity = v ) ; } ;


int OCTET_STRING_DLL Octet_String::verbosity = 0 ;


bool
Octet_String::store ( FILE * the_persistent_storage_output_file , unsigned short & checksum ) const
  {
    bool result = true ;

    result &=
      1 == fwrite ( & the_start , sizeof ( the_start ) , 1 , the_persistent_storage_output_file ) ;
    checksum ^= the_start ;

    result &=
      1 == fwrite ( & the_stop , sizeof ( the_stop ) , 1 , the_persistent_storage_output_file ) ;
    checksum ^= the_stop ;

    result &= the_storage . store ( the_persistent_storage_output_file , checksum ) ;

    return result ;

  } ;

bool 
Octet_String::load ( FILE * the_persistent_storage_input_file , unsigned short & checksum )
  {
    bool result = true ;

    result &=
      1 == fread ( & the_start , sizeof ( the_start ) , 1 , the_persistent_storage_input_file ) ;
    checksum ^= the_start ;

    result &=
      1 == fread ( & the_stop , sizeof ( the_stop ) , 1 , the_persistent_storage_input_file ) ;
    checksum ^= the_stop ;

    result &= the_storage . load ( the_persistent_storage_input_file , checksum ) ;

    return result ;

  } ;
