// File Manager
//
// COPYRIGHT
// ---------
// The software was developed by the National Institute of Standards and
// Technology (NIST), an agency of the Federal Government.  Pursuant to
// Title 15 United States Code Section 105, works of NIST are not subject
// to copyright protection in the United States and are considered to be
// in the public domain.  As a result, a formal license is not needed to
// use the software.
//
// The Government Smart Card Reference Implementation software is provided
// by NIST as a service and is expressly provided "AS IS."
// NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY,
// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.
// NIST does not warrant or make any representations regarding the use of
// the software or the results thereof, including but not limited to the
// correctness, accuracy, reliability or usefulness of the software.
//
// Permission to use this software is contingent upon your acceptance of
// the terms of this agreement and upon your providing appropriate
// acknowledgments of NIST's ownership of the software.

#if defined(_WIN32)
#include <windows.h>
#endif
#include <File_Manager.h>
#include <macros.h>
#include <Application.h>
#include <PIV_Card_Application.h>

#include "File_System_Initial_Configuration.inc"

bool build_test_filesystem_when_initializing = false ;

void
File_Manager::create_initial_file_system ( )
  {
    File_System * file_system ;
    file_system = new File_System ( MF_FCP , sizeof ( MF_FCP ) ) ;

    file_system -> select_file ( MF, 0x00 );

    //
    // Create the SP 800-73 Data Objects in the MF
    //
    // BER-TLV Tag: 5FC101
    file_system -> create_data_object    ( Octet_String ( CARD_X509_FCP ,  sizeof ( CARD_X509_FCP ) ) ) ;
    file_system -> put_data( BER_TLV_Tag ( Octet_String ( CARD_X509_Tag ,  sizeof ( CARD_X509_Tag ) ) ) ,
                                           Octet_String ( CARD_X509_DATA , sizeof ( CARD_X509_DATA ) ) ) ;

    // BER-TLV Tag: 5FC102
    file_system -> create_data_object    ( Octet_String ( CHUID_FCP ,  sizeof ( CHUID_FCP ) ) ) ;
    file_system -> put_data( BER_TLV_Tag ( Octet_String ( CHUID_Tag ,  sizeof ( CHUID_Tag ) ) ) ,
                                           Octet_String ( CHUID_DATA , sizeof ( CHUID_DATA ) ) ) ;

    // BER-TLV Tag: 5FC103
    file_system -> create_data_object    ( Octet_String ( FINGER_1_FCP ,  sizeof ( FINGER_1_FCP ) ) ) ;
    file_system -> put_data( BER_TLV_Tag ( Octet_String ( FINGER_1_Tag ,  sizeof ( FINGER_1_Tag ) ) ) ,
                                           Octet_String ( FINGER_1_DATA , sizeof ( FINGER_1_DATA ) ) ) ;

    // BER-TLV Tag: 5FC104
    file_system -> create_data_object (    Octet_String ( FINGER_2_FCP ,  sizeof ( FINGER_2_FCP ) ) ) ;
    file_system -> put_data( BER_TLV_Tag ( Octet_String ( FINGER_2_Tag ,  sizeof ( FINGER_2_Tag ) ) ) ,
                                           Octet_String ( FINGER_2_DATA , sizeof ( FINGER_2_DATA ) ) ) ;

    // BER-TLV Tag: 5FC105
    file_system -> create_data_object    ( Octet_String ( PIV_X509_FCP ,  sizeof ( PIV_X509_FCP ) ) ) ;
    file_system -> put_data( BER_TLV_Tag ( Octet_String ( PIV_X509_Tag ,  sizeof ( PIV_X509_Tag ) ) ) ,
                                           Octet_String ( PIV_X509_DATA , sizeof ( PIV_X509_DATA ) ) ) ;

    // BER-TLV Tag: 5FC106
    file_system -> create_data_object    ( Octet_String ( SECURITY_OBJECT_FCP ,  sizeof ( SECURITY_OBJECT_FCP ) ) ) ;
    file_system -> put_data( BER_TLV_Tag ( Octet_String ( SECURITY_OBJECT_Tag ,  sizeof ( SECURITY_OBJECT_Tag ) ) ) ,
                                           Octet_String ( SECURITY_OBJECT_DATA , sizeof ( SECURITY_OBJECT_DATA ) ) ) ;

    // BER-TLV Tag: 5FC107
    file_system -> create_data_object    ( Octet_String ( CCC_FCP ,  sizeof ( CCC_FCP ) ) ) ;
    file_system -> put_data( BER_TLV_Tag ( Octet_String ( CCC_Tag ,  sizeof ( CCC_Tag ) ) ) ,
                                           Octet_String ( CCC_DATA , sizeof ( CCC_DATA ) ) ) ;

    // BER-TLV Tag: 5FC108
    file_system -> create_data_object    ( Octet_String ( FACIAL_IMAGE_FCP ,  sizeof ( FACIAL_IMAGE_FCP ) ) ) ;
    file_system -> put_data( BER_TLV_Tag ( Octet_String ( FACIAL_IMAGE_Tag ,  sizeof ( FACIAL_IMAGE_Tag ) ) ) ,
                                           Octet_String ( FACIAL_IMAGE_DATA , sizeof ( FACIAL_IMAGE_DATA ) ) ) ;

    // BER-TLV Tag: 5FC109
    file_system -> create_data_object    ( Octet_String ( PRINTED_INFO_FCP ,  sizeof ( PRINTED_INFO_FCP ) ) ) ;
    file_system -> put_data( BER_TLV_Tag ( Octet_String ( PRINTED_INFO_Tag ,  sizeof ( PRINTED_INFO_Tag ) ) ) ,
                                           Octet_String ( PRINTED_INFO_DATA , sizeof ( PRINTED_INFO_DATA ) ) ) ;

    // BER-TLV Tag: 5FC10A
    file_system -> create_data_object    ( Octet_String ( SIGNING_X509_FCP ,  sizeof ( SIGNING_X509_FCP ) ) ) ;
    file_system -> put_data( BER_TLV_Tag ( Octet_String ( SIGNING_X509_Tag ,  sizeof ( SIGNING_X509_Tag ) ) ) ,
                                           Octet_String ( SIGNING_X509_DATA , sizeof ( SIGNING_X509_DATA ) ) ) ;

    // BER-TLV Tag: 5FC10B
    file_system -> create_data_object    ( Octet_String ( KEY_MANAGEMENT_X509_FCP ,  sizeof ( KEY_MANAGEMENT_X509_FCP ) ) ) ;
    file_system -> put_data( BER_TLV_Tag ( Octet_String ( KEY_MANAGEMENT_X509_Tag ,  sizeof ( KEY_MANAGEMENT_X509_Tag ) ) ) ,
                                           Octet_String ( KEY_MANAGEMENT_X509_DATA , sizeof ( KEY_MANAGEMENT_X509_DATA ) ) ) ;

    the_security_system_pointer =
      file_system -> the_security_system_pointer =
        Security_System_Pointer::object_pointer_to_indexed_pointer ( new Security_System ( ) ) ;

    // Create the default PIN (key reference 0x80) with its PUK (key reference 0x81)
    PIN_Pointer the_default_PIN_pointer =
      PIN_Pointer::object_pointer_to_indexed_pointer
        ( new PIN ( the_default_PIN , sizeof ( the_default_PIN ) ) ) ;

    PIN_Pointer the_default_PUK_pointer =
      PIN_Pointer::object_pointer_to_indexed_pointer
        ( new PIN ( the_default_PUK , sizeof ( the_default_PUK ) ) ) ;

    PIN_Entry * the_default_PIN_entry =
      new PIN_Entry ( the_default_PIN_pointer ,
                      the_default_PUK_pointer,
                      the_default_PIN_retries ,
                      the_default_PIN_retries ,
                      the_default_PUK_retries
                      ) ;

    the_security_system_pointer
      -> the_key_store . put_key_entry_pointer
            ( the_default_PIN_reference ,
              PIN_Entry_Pointer::object_pointer_to_indexed_pointer ( the_default_PIN_entry ) ) ;


    // Create the PIV Card Application Validation Key (key reference 0x9A)
    RSA_Key_Pointer the_default_RSA_key_pointer =
      RSA_Key_Pointer::object_pointer_to_indexed_pointer
        ( new RSA_Key ( the_default_RSA_key_n , sizeof ( the_default_RSA_key_n ) ,
                        the_default_RSA_key_d , sizeof ( the_default_RSA_key_d ) ,
                        the_default_RSA_key_e , sizeof ( the_default_RSA_key_e ) ) ) ;

    RSA_Key_Entry * the_default_RSA_key_entry =
      new RSA_Key_Entry ( the_default_RSA_key_pointer ) ;

    the_security_system_pointer
      -> the_key_store . put_key_entry_pointer
            ( the_default_RSA_key_reference ,
              RSA_Key_Entry_Pointer::object_pointer_to_indexed_pointer ( the_default_RSA_key_entry ) ) ;


    // Create the PIV Card Application Adminstration Key (key reference 0x9B)
    Symmetric_Key_Pointer the_default_symmetric_key_pointer_1 =
      Symmetric_Key_Pointer::object_pointer_to_indexed_pointer
        ( new Symmetric_Key (the_default_symmetric_key ,
                              sizeof ( the_default_symmetric_key ) ) ) ;

    Symmetric_Key_Entry * the_default_symmetric_key_entry_1 =
      new Symmetric_Key_Entry ( the_default_symmetric_key_pointer_1 ) ;

    the_security_system_pointer
      -> the_key_store . put_key_entry_pointer
            ( the_default_symmetric_key_reference_1 ,
              Symmetric_Key_Entry_Pointer::object_pointer_to_indexed_pointer ( the_default_symmetric_key_entry_1 ) ) ;


    // Create the PIV Card Authentication Key (key reference 0x9E)
    Symmetric_Key_Pointer the_default_symmetric_key_pointer_2 =
      Symmetric_Key_Pointer::object_pointer_to_indexed_pointer
        ( new Symmetric_Key (the_default_symmetric_key ,
                              sizeof ( the_default_symmetric_key ) ) ) ;

    Symmetric_Key_Entry * the_default_symmetric_key_entry_2 =
      new Symmetric_Key_Entry ( the_default_symmetric_key_pointer_2 ) ;

    the_security_system_pointer
      -> the_key_store . put_key_entry_pointer
            ( the_default_symmetric_key_reference_2 ,
              Symmetric_Key_Entry_Pointer::object_pointer_to_indexed_pointer ( the_default_symmetric_key_entry_2 ) ) ;


    the_file_system_pointer =
      File_System_Pointer::object_pointer_to_indexed_pointer ( file_system ) ;
  } ;


long File_Manager::initialize ( int v )
{
  File_Manager::set_verbosity ( v ) ;
  if ( File_System_Pointer ( ) == the_file_system_pointer )
    create_initial_file_system ( ) ;
  return 0 ;
}

long File_Manager::finalize ( )
{
  return 0 ;
}

long File_Manager::store_load_module ( const unsigned long flags,
                                       const Octet_String & module_AID,
                                       const Load_Module & data_blocks )
{
  if ( data_blocks . empty ( ) )
    return 1 ;

  Load_Module::const_iterator p = data_blocks.begin ( ) ;

  Octet_String DLL_name = * p ;

  p ++ ;

  if ( p != data_blocks . end ( ) )
    return 1 ;

  the_AID_to_DLL_name_map . insert
    ( AID_to_DLL_Name_Map::value_type ( module_AID , DLL_name ) ) ;

  return 0 ;
}


long File_Manager::create_runnable_instance ( const unsigned long flags ,
                                              const Octet_String & module_AID ,
                                              const Octet_String & requested_instance_AID ,
                                              const Octet_String & install_parameters ,
                                              Octet_String & created_instance_AID )
  {
    long result = connect_runnable_instance ( module_AID ) ;
    if ( 0 == result )
      created_instance_AID = requested_instance_AID ;
    return result ;
  } ;


long File_Manager::connect_runnable_instance ( const Octet_String & module_AID )
  {

    Octet_String DLL_name = find_DLL_for_AID ( module_AID );
    size_t DLL_name_length = DLL_name . length ( ) ;

    if ( 0 == DLL_name_length )
      throw RAPDU_Packet::SW_INCORRECT_P1_P2 ;

#ifdef _DEBUG	// This block assumes the DLL_file_name only needs a 'D' appended to the end
	DLL_name_length += 1;
#endif
    char * DLL_file_name =
      reinterpret_cast < char * > ( malloc ( DLL_name_length + 1 ) ) ;
    const Octet * DLL_name_data = DLL_name . data ( ) ;

#ifdef _DEBUG	// This block assumes the DLL_file_name only needs a 'D' appended to the end
	for ( size_t i = 0 ; i < DLL_name_length - 1 ; i ++ )
#else
	for ( size_t i = 0 ; i < DLL_name_length ; i ++ )
#endif
      DLL_file_name [ i ] = DLL_name_data [ i ] ;

#ifdef _DEBUG	// This block assumes the DLL_file_name only needs a 'D' appended to the end
	DLL_file_name [ DLL_name_length - 1 ] = 'D';
#endif
    DLL_file_name [ DLL_name_length ] = '\0' ;

    HINSTANCE hDLL ;
    hDLL = LoadLibrary ( DLL_file_name ) ;
    free ( DLL_file_name ) ;

    if ( (HINSTANCE)NULL == hDLL )
      return 1 ;

    Application_Creator * application_creator_pointer =
      ( Application_Creator * ) GetProcAddress ( hDLL , "create_application" ) ;

    if ( ( Application_Creator * ) 0 == application_creator_pointer )
      return 1 ;

    Application * application_pointer =
      ( * application_creator_pointer) ( this ,
                                          the_file_system_pointer ,
                                          the_security_system_pointer ,
                                          the_challenge_pointer ) ;


    if ( ( Application * ) 0 == application_pointer )
      return 1 ;

    const Octet_String & application_AID = application_pointer -> AID ( ) ;

    if ( ! ( application_AID == module_AID ) )
      return 1 ;

    return 0 ;

  } ;


void
File_Manager::get_ATR ( unsigned char * ATR ,
                        unsigned int & ATR_length )
{
  static Octet the_FM_ATR [ ] = { 0x3B , 0xF0 , 0x11 , 0x00 , 0xFF , 0x00 } ;
  ATR_length = sizeof ( the_FM_ATR ) ;
  memcpy ( ATR , the_FM_ATR , sizeof ( the_FM_ATR ) ) ;
}



long File_Manager::exchange_plaintext_APDUs ( const CAPDU_Packet & command, RAPDU_Packet & response )
  {
    RAPDU_Packet::Status_Word status = RAPDU_Packet::SW_FUNCTION_NOT_SUPPORTED ;
    long result = 0 ;
    response . set_APDU_length ( 0 ) ;

    // Dispatch on the INS
    CLAINS clains = command.get_CLAINS ( );

    // Hardwire dispatch to PIV Card Application 
    Dispatch_Map::const_iterator application_pointer_pointer =
      the_dispatch_map . find ( CAPDU_Packet::CLAINS_SELECT ) ;

    if ( application_pointer_pointer != the_dispatch_map . end (  ) )
      {
        try
          {
            result = application_pointer_pointer -> second -> exchange_APDUs ( command, response ) ;
            response . set_status ( RAPDU_Packet::SW_OK ) ;
          }
        catch ( RAPDU_Packet::Status_Word sw )
          {
            response . set_status ( sw ) ;
          }
      }
    else
      response . set_status ( RAPDU_Packet::SW_FUNCTION_NOT_SUPPORTED ) ;

    return result ;
  } ;



long File_Manager::exchange_APDUs ( const CAPDU_Packet & command, RAPDU_Packet & response )
  {
    RAPDU_Packet::Status_Word status = RAPDU_Packet::SW_UNKNOWN ;
    long result = 0 ;
    response . set_APDU_length ( 0 ) ;

    try // catch anything otherwise uncaught and turn it into a 6F00 status
      {
        // Dispatch on the INS
        CLAINS clains = command.get_CLAINS ( );

        if ( CAPDU_Packet::CLAINS_SECURE_MESSAGING_FLAG
              == ( clains & CAPDU_Packet::CLAINS_SECURE_MESSAGING_MASK ) )
          {
            // Secure messaging
            Security_System & the_security_system =
              * ( the_file_system_pointer -> the_security_system_pointer  ) ;
            Octet P3 = command . get_P3 ( ) ;
            // initial sanity check
            if ( ! ( 1 + 1 + 4 == P3  // checksum TLV
                     || 1 + 1 + 4 + 1 + 1 < P3 ) ) // also payload TLV
              throw RAPDU_Packet::SW_SM_DATA_OBJECT_MISSING ;

            CAPDU_Packet plaintext_command  ;
            RAPDU_Packet plaintext_response ;
            plaintext_command
              . put_APDU_u16
                  ( CAPDU_Packet::OFFSET_CLA ,
                    clains & ~CAPDU_Packet::CLAINS_SECURE_MESSAGING_MASK ) ;
            plaintext_command
              . put_APDU_u16
                  ( CAPDU_Packet::OFFSET_P1 ,
                    command . get_P1P2 ( ) ) ;

            Key_Reference default_key_reference ( 0 ) ;
            Symmetric_Key & key =
              the_security_system
                . validate_algorithm_and_symmetric_key ( Algorithm_Identifier::Triple_DES_ECB_128 ,
                                                        default_key_reference  ) ;
            try
              {
                Octet_String command_payload ( command . payload ( ) ,
                                              command . get_P3 ( ) ) ;
                BER_TLV_Sequence secure_payload
                  ( command_payload ,
                    command_payload . start ( ) ,
                    command_payload . stop ( ) ) ;

                Primitive_BER_TLV_Pointer checksum_pointer =
                  secure_payload . find ( SM_CRYPTOGRAPHIC_CHECKSUM ) ;

                if ( Primitive_BER_TLV_Pointer ( ) == checksum_pointer
                       || 4 != checksum_pointer -> values ( ) . length ( ) )
                  throw RAPDU_Packet::SW_SM_DATA_OBJECT_INCORRECT ;

                Octet_String cyphertext ( 8 ) ;

                Memory::size_t plaintext_payload_buffer_length = P3 - 6 ;
                if ( 0 == plaintext_payload_buffer_length )
                  plaintext_payload_buffer_length = 8 ;
                else
                  plaintext_payload_buffer_length = ( ( plaintext_payload_buffer_length  + 7 ) / 8 ) * 8 ;

                Octet_String plaintext_payload_buffer ( plaintext_payload_buffer_length ) ;
                memset ( plaintext_payload_buffer . data ( ) , 0 , plaintext_payload_buffer_length ) ;

                if ( 1 + 1 + 4 + 1 + 1 < P3 ) // checksum and payload TLV
                  {

                    Primitive_BER_TLV_Pointer plaintext_payload_pointer =
                      secure_payload . find ( SM_PLAIN_VALUE_NOT_BER_TLV_CHK ) ;

                    Octet plaintext_length =
                      static_cast < Octet >
                          ( plaintext_payload_pointer -> values ( ) . length ( ) ) ;

                    plaintext_command . set_APDU_length ( CAPDU_Packet::OFFSET_CDATA
                                                          + plaintext_length  ) ;

                    plaintext_command . put_APDU_octet ( CAPDU_Packet::OFFSET_P3 ,
                                                        plaintext_length ) ;

                    Octet_String::cpy ( plaintext_command . APDU ( )
                                          + CAPDU_Packet::OFFSET_CDATA ,
                                        plaintext_length ,
                                        plaintext_payload_pointer -> values ( ) ) ;

                    Octet_String::cpy ( plaintext_payload_buffer ,
                                        * plaintext_payload_pointer  ) ;

                  }

                the_security_system . Triple_DES_ECB_128_encrypt ( key , plaintext_payload_buffer , cyphertext ) ;

                if ( ! ( checksum_pointer -> values ( ) == Octet_String ( cyphertext , 0 , 4 ) ) )
                  throw RAPDU_Packet::SW_SM_DATA_OBJECT_INCORRECT ;

                result = exchange_plaintext_APDUs ( plaintext_command  ,
                                                    plaintext_response ) ;

                // If the response were also encrypted, it could be decrypted here.

                response = plaintext_response ;
              }
            catch ( Octet_String_Index_Exception )
              {
                throw RAPDU_Packet::SW_SM_DATA_OBJECT_MISSING ;
              }
            catch ( Malformed_TLV_Exception )
              {
                throw RAPDU_Packet::SW_SM_DATA_OBJECT_MISSING ;
              }
            catch ( Wrong_BER_TLV_Pointer_Type_Exception )
              {
                throw RAPDU_Packet::SW_SM_DATA_OBJECT_MISSING ;
              }
          }
        else
          result = exchange_plaintext_APDUs ( command , response ) ;
      }
    catch ( ... )
      {
        debug_message ( "Unexpected exception -- responding with 0x%04X\n", RAPDU_Packet::SW_UNKNOWN );
#if defined(_WIN32) && defined(_DEBUG)
        DebugBreak();
#endif
        response . set_status ( RAPDU_Packet::SW_UNKNOWN ) ;
      }

    if ( 1 < the_verbosity )
      the_file_system_pointer -> dump ( ) ;

    return result ;
  } ;


long File_Manager :: power_up ( void )
  {
    if ( 1 < the_verbosity )
      if ( File_System_Pointer ( ) != the_file_system_pointer )
        the_file_system_pointer -> dump ( ) ;

    return 0 ;
  } ;


long File_Manager :: power_down ( void )
  {
    return 0 ;
  } ;



bool
File_Manager::store_persistent_state ( FILE * the_persistent_storage_output_file , unsigned short & checksum )
  {
    bool result = true ;

    result &= the_file_system_pointer . store ( the_persistent_storage_output_file , checksum ) ;

    unsigned short int the_AID_to_DLL_name_map_size =
      static_cast < unsigned short int >
          ( the_AID_to_DLL_name_map . size ( ) ) ;

    result &=
      1 == fwrite ( & the_AID_to_DLL_name_map_size ,
                    sizeof ( the_AID_to_DLL_name_map_size  ) ,
                    1 ,
                    the_persistent_storage_output_file ) ;

    checksum ^= the_AID_to_DLL_name_map_size ;

    for ( AID_to_DLL_Name_Map::const_iterator
            p     = the_AID_to_DLL_name_map . begin ( ) ,
            p_end = the_AID_to_DLL_name_map . end ( ) ;
          p != p_end ;
          p ++ )
      {
        result &= p -> first . store ( the_persistent_storage_output_file , checksum ) ;
        result &= p -> second . store ( the_persistent_storage_output_file , checksum ) ;
      }

    return result ;
  } ;

bool
File_Manager::load_persistent_state ( FILE * the_persistent_storage_input_file , unsigned short & checksum )
  {
    bool result = true ;

    result &= the_file_system_pointer . load ( the_persistent_storage_input_file , checksum ) ;

    if ( File_System_Pointer ( ) != the_file_system_pointer )
      the_security_system_pointer =
        the_file_system_pointer -> the_security_system_pointer ;

    unsigned short int the_AID_to_DLL_name_map_size ;

    result &=
      1 == fread ( & the_AID_to_DLL_name_map_size ,
                   sizeof ( the_AID_to_DLL_name_map_size  ) ,
                   1 ,
                   the_persistent_storage_input_file ) ;

    checksum ^= the_AID_to_DLL_name_map_size ;

    for ( unsigned short int i = 0 ; i < the_AID_to_DLL_name_map_size ; i ++ )
      {
        Octet_String AID, DLL_name ;

        result &= AID . load ( the_persistent_storage_input_file , checksum ) ;
        result &= DLL_name . load ( the_persistent_storage_input_file , checksum ) ;

        the_AID_to_DLL_name_map . insert
          ( AID_to_DLL_Name_Map::value_type ( AID , DLL_name ) ) ;

        connect_runnable_instance ( AID ) ;

      }

    return result ;
  } ;

