// Java Card Manager
//
// COPYRIGHT
// ---------
// The software was developed by the National Institute of Standards and 
// Technology (NIST), an agency of the Federal Government.  Pursuant to 
// Title 15 United States Code Section 105, works of NIST are not subject
// to copyright protection in the United States and are considered to be
// in the public domain.  As a result, a formal license is not needed to
// use the software.
// 
// The Government Smart Card Reference Implementation software is provided
// by NIST as a service and is expressly provided "AS IS."  
// NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.
// NIST does not warrant or make any representations regarding the use of
// the software or the results thereof, including but not limited to the
// correctness, accuracy, reliability or usefulness of the software.
// 
// Permission to use this software is contingent upon your acceptance of 
// the terms of this agreement and upon your providing appropriate
// acknowledgments of NIST's ownership of the software.

#include <Java_Card_Manager.h>

Java_Card_Manager::Java_Card_Manager ( ) 
  : Application_Manager ( ) 
  {
    debug_message ( "In Java_Card_Manager constructor.\n" );
  } ;

long Java_Card_Manager::initialize ( int v )
{
  long result ;
  the_tlp_apdu_server.set_verbosity ( v ) ;

  result = the_tlp_apdu_server . set_hostname ( "localhost" ) ;
  if ( 0 != result ) 
    {
      error_message ( "set_hostname failed." ) ;
      return result ;
    };

  the_tlp_apdu_server.set_port ( 9026 );
  
  debug_message ( "Initializing.\n" ) ;
  the_tlp_apdu_server.report_initialization_errors = false ;
  result = the_tlp_apdu_server . initialize ( v )  ;
  if ( 0 != result ) 
    {
      
      if ( 0 == strnicmp("localhost",the_tlp_apdu_server.get_hostname(),9))
        {
          debug_message ( "... initializing failed, but before giving up, try starting the cref ...\n" ) ;
          char command_buffer [ 32 ] ;
          sprintf ( command_buffer , "start cref -p %d" , the_tlp_apdu_server . get_port ( ) ) ;
          debug_message ( "calling system ( \"%s\" )\n" , command_buffer ) ;
          system ( command_buffer ) ;
          Sleep ( 1000 ) ;
          the_tlp_apdu_server.report_initialization_errors = true ;
          result = the_tlp_apdu_server . initialize ( v )  ;
        }
      if ( 0 != result ) 
        {
          error_message ( "initialize failed." ) ;
          return result ;
        };
    };

  return 0 ;
}

long Java_Card_Manager::exchange_APDUs ( const CAPDU_Packet & command, RAPDU_Packet & response )
{
  long result ;
  command . log ( "Java_Card_Manager::exchange_APDUs:CAPDU_Packet: " ) ;
  result = the_tlp_apdu_server . exchange_APDUs ( command , response ) ;
  response . log ( "Java_Card_Manager::exchange_APDUs:RAPDU_Packet: ") ;
  return result ;
}

long Java_Card_Manager::finalize ( )
{
  long result ;
  
  debug_message ( "Finalizing.\n" ) ;
  result = the_tlp_apdu_server . finalize ( ) ;
  if ( 0 != result )
    {
      error_message ( "Finalize failed." ) ;
      return result ;
    };
  
  return result ;
}

const Octet Java_Card_Manager::select_OCI_command [ ] = 
  { 
    // Select by AID and AID length
    CLA_INS ( CAPDU_Packet::CLAINS_SELECT ) , 0x04 , 0x00 , 0x09 , 
    // OC Install AID
    0xA0 , 0x00 , 0x00 , 0x00 , 0x62 , 0x03 , 0x01 , 0x08 , 0x01
  } ;


long Java_Card_Manager::store_load_module( const unsigned long flags,
                                           const Octet_String & AID ,
                                           const Load_Module & data_blocks )
{
  CAPDU_Packet command ;
  RAPDU_Packet response ;
  long result = 0 ;

// Select the On-Card Installer (OCI) Applet
// 0x00 0xA4 0x04 0x00 0x09 0xa0 0x00 0x00 0x00 0x62 0x03 0x01 0x08 0x01 0x00;
// SW1: 90, SW2: 00

  command . set_APDU ( select_OCI_command , sizeof ( select_OCI_command ) ) ;
  result = exchange_APDUs ( command , response ) ;
  if ( 0 != result )
    return result ;
  result = ! ( ( 2 <= response . get_APDU_length ( ) )
              &&( 0x9000 == response . get_APDU_u16 ( response . get_APDU_length ( ) - 2 ) ) ) ;
  if ( 0 != result )
    return result ;


  for ( Load_Module::const_iterator p = data_blocks . begin ( ) ;
        p != data_blocks . end ( ) ;
        p ++ )
    {
      APDU ( * p ) . store_into_APDU_packet ( command ) ;
      result = exchange_APDUs ( command , response ) ;
      if ( 0 != result )
        return result ;
      result = ! ( ( 2 <= response . get_APDU_length ( ) )
                 &&( 0x9000 == response . get_APDU_u16 ( response . get_APDU_length ( ) - 2 ) ) ) ;
      if ( 0 != result )
        return result ;
    }

  return result ;
}

long Java_Card_Manager::create_runnable_instance ( const unsigned long flags ,
                                                   const Octet_String & module_AID ,
                                                   const Octet_String & requested_instance_AID ,
                                                   const Octet_String & install_parameters ,
                                                   Octet_String & created_instance_AID )
{
  CAPDU_Packet command ;
  RAPDU_Packet response ;
  long result = 0 ;

  command . set_APDU ( select_OCI_command , sizeof ( select_OCI_command ) ) ;
  result = exchange_APDUs ( command , response ) ;
  if ( 0 != result )
    return result ;
  result = ! ( ( 2 <= response . get_APDU_length ( ) )
              &&( 0x9000 == response . get_APDU_u16 ( response . get_APDU_length ( ) - 2 ) ) ) ;
  if ( 0 != result )
    return result ;

  // Create the applet instance -- the applet module_AID is the parameter module_AID

  // Make up the command APDU Octet_String
  Octet payload_length = static_cast < Octet > ( 1 + module_AID . length ( ) 
                                               + 1 
                                               + 1 + requested_instance_AID . length ( ) 
                                               + 1 + install_parameters . length () ) ;

  Octet_String create_applet_instance ( CAPDU_Packet::OFFSET_CDATA  + payload_length  ) ;

  const Octet create_command_header [ ] = { CLA_INS ( CLAINS_CREATE_APPLET_INSTANCE )  , 0x00 , 0x00  } ;

  // Copy in the header
  Octet_String::cpy ( create_applet_instance , create_command_header , sizeof ( create_command_header ) );
  // Set the length byte as P3
  create_applet_instance . put_octet ( CAPDU_Packet::OFFSET_P3 , payload_length ) ;

  // Set the module_AID length byte
  create_applet_instance . put_octet ( CAPDU_Packet::OFFSET_CDATA + 0 , 
                                       static_cast < Octet > ( module_AID . length ( ) ) ) ;
  // Copy in the module_AID
  Octet_String::cpy ( Octet_String ( create_applet_instance , 
                                     CAPDU_Packet::OFFSET_CDATA + 1  ) , 
                      module_AID ) ;

  // Set the installation parameters length byte
  create_applet_instance . put_octet ( CAPDU_Packet::OFFSET_CDATA + 1 + module_AID . length ( ) , 
                                       static_cast < Octet > ( 1 + requested_instance_AID . length ( ) + 
                                                               1 + install_parameters . length ( ) ) ) ;
  // Set the requested_instance_AID length byte
  create_applet_instance . put_octet ( CAPDU_Packet::OFFSET_CDATA + 1 + module_AID . length ( ) + 1 , 
                                       static_cast < Octet > ( requested_instance_AID . length ( ) ) ) ;
  // Copy in the requested_instance_AID
  Octet_String::cpy ( Octet_String ( create_applet_instance , 
                                     CAPDU_Packet::OFFSET_CDATA + 1 + module_AID . length ( ) + 1 + 1 ) , 
                      requested_instance_AID ) ;

  // Set the install parameters length byte
  create_applet_instance . put_octet ( CAPDU_Packet::OFFSET_CDATA + 1 + module_AID . length ( ) + 1 + 1 + requested_instance_AID . length ( ) , 
                                       static_cast < Octet > ( install_parameters . length ( ) ) ) ;

  // Copy in the install parameters
  Octet_String::cpy ( Octet_String ( create_applet_instance , 
                                     CAPDU_Packet::OFFSET_CDATA + 1 + module_AID . length ( ) + 1 + 1 + requested_instance_AID . length ( ) + 1 ) , 
                      install_parameters ) ;

  Memory::size_t create_applet_instance_length = CAPDU_Packet::OFFSET_CDATA + payload_length ;

// Create Applet
  command . set_APDU_length ( create_applet_instance_length ) ;
  Octet_String::cpy ( command . APDU ( ) , 
                      static_cast < Memory::size_t > 
                          ( command . the_frame_size - command . header_length ) , 
                      create_applet_instance ,
                      create_applet_instance_length ) ;
  result = exchange_APDUs ( command , response ) ;

  if ( 0 != result )
    return result ;
  result = ! ( ( 2 <= response . get_APDU_length ( ) )
              &&( 0x9000 == response . get_APDU_u16 ( response . get_APDU_length ( ) - 2 ) ) ) ;
  if ( 0 != result )
    return result ;


  // Return the applet instance module_AID
  created_instance_AID = 
    Octet_String ( response . payload ( ) , 
                   static_cast < Memory::size_t > 
                       ( response . get_APDU_length ( ) - 2 ) ) ;

  return result ;
}

long Java_Card_Manager::connect_runnable_instance ( const Octet_String & module_AID )
  {
    return 1 ;
  }


long Java_Card_Manager::power_up         ( void )
{
  long result = 0 ;
  debug_message ( "Powering up cref.\n" ) ;
  result = the_tlp_apdu_server . power_up ( )  ;
  if ( 0 != result )
    error_message ( "power_up failed." ) ;
  return result ;
}


long Java_Card_Manager::power_down       ( void )
{
  long result = 0 ;
  debug_message ( "Powering down cref.\n" ) ;
  result = the_tlp_apdu_server . power_down ( )  ;
  if ( 0 != result )
    error_message ( "Power down failed." ) ;
  return result ;
}
