// PIV Card Application
//
// COPYRIGHT
// ---------
// The software was developed by the National Institute of Standards and 
// Technology (NIST), an agency of the Federal Government.  Pursuant to 
// Title 15 United States Code Section 105, works of NIST are not subject
// to copyright protection in the United States and are considered to be
// in the public domain.  As a result, a formal license is not needed to
// use the software.
// 
// The Government Smart Card Reference Implementation software is provided
// by NIST as a service and is expressly provided "AS IS."  
// NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.
// NIST does not warrant or make any representations regarding the use of
// the software or the results thereof, including but not limited to the
// correctness, accuracy, reliability or usefulness of the software.
// 
// Permission to use this software is contingent upon your acceptance of 
// the terms of this agreement and upon your providing appropriate
// acknowledgments of NIST's ownership of the software.

#include <windows.h>
#include <crtdbg.h>

#include "PIV_Card_Application.h"
#include "macros.h"

#define T0

//
// The PIV card application is notified that it is being selected
//
long PIV_Card_Application::select_application ( const CAPDU_Packet & command, RAPDU_Packet & response )
  {
    if ( 0x04 != command . get_P1 ( ) ||  0x00 != command . get_P2 ( ))
        throw RAPDU_Packet::SW_INCORRECT_P1_P2 ;

#if !defined(T0)
	// SELECT must be a Case 4 command
	if ( ( command . get_P3 ( ) + 5 ) != ( command . get_APDU_length ( ) - 1 ) )
		throw RAPDU_Packet::SW_WRONG_PARAMETERS_IN_DATA_FIELD ;
#endif

	if ( 0 != command . get_Le ( ) )
	{
		Octet_String the_application_properties ( PIVCardApplicationProperties , command . get_Le ( ) );
		APDU ( the_application_properties ) . store_into_APDU_packet ( response ) ;
	}
	else
	{
		Octet_String the_application_properties ( PIVCardApplicationProperties , sizeof PIVCardApplicationProperties );
		APDU ( the_application_properties ) . store_into_APDU_packet ( response ) ;
	}

    return 0 ;
  } ;

//
// GET DATA
//
long PIV_Card_Application::get_data ( const CAPDU_Packet & command, RAPDU_Packet & response )
{
    Octet_String the_response ;

    if ( 0x3F != command . get_P1 ( ) ||  0xFF != command . get_P2 ( ))
        throw RAPDU_Packet::SW_INCORRECT_P1_P2 ;

#if !defined(T0)	
	// GET DATA must be a Case 4 command
	if ( ( command . get_P3 ( ) + 5 ) != ( command . get_APDU_length ( ) - 1 ) )
		throw RAPDU_Packet::SW_WRONG_PARAMETERS_IN_DATA_FIELD ;
#endif

    Primitive_BER_TLV_Pointer the_tag_list = 
        BER_TLV::create ( Octet_String ( command . payload ( ) , command . get_P3 ( ) ) , 0 ) ;

    if ( DATA_FIELD_TAG_LIST != the_tag_list -> tag ( ) . get_octet ( 0 ) ||
         3 != the_tag_list -> length ( ) . get_octet ( 0 ))
        throw RAPDU_Packet::SW_WRONG_PARAMETERS_IN_DATA_FIELD ;

    BER_TLV_Tag the_tag ( the_tag_list -> values( ) );

	if( (the_security_system_pointer -> contactless_interface ( ) ) && ( ( the_tag . get_octet(2) != ( TAG_CHUID & 0x0000FF ) ) && ( the_tag.get_octet(2) != ( TAG_PIV_APP_X509 & 0x0000FF ) ) ) )
        throw RAPDU_Packet::SW_SECURITY_STATUS_NOT_SATISFIED ;
 
	//Remove PIN protection for PIV Certificate, X509 Digital Sign and X509 Key management
	if((the_tag.get_octet(2) != (TAG_LOGICAL_X509 & 0x0000FF)) && 
		(the_tag.get_octet(2) != (TAG_SIGNING_X509 & 0x0000FF)) && 
		(the_tag.get_octet(2) != (TAG_KEY_X509 & 0x0000FF))) 
			the_file_system_pointer -> verify_data_object_authorization ( the_tag , ACR_AM_DO_GET_DATA ) ;

	//Obsolete with the change above!!
	//if( ( the_tag . get_octet(2) == ( TAG_SIGNING_X509 & 0x0000FF ) ) &&  ! PIN_Always )
    //    throw RAPDU_Packet::SW_SECURITY_STATUS_NOT_SATISFIED ;

    Octet_String the_data_object ;

    the_file_system_pointer -> get_data ( the_tag , the_data_object ) ;

    // Return the retrieved data
    Octet_String the_tlv ( MAXIMUM_GET_DATA ) ;
    Octet_String where ( the_tlv) ;
    //NOTE NIST
    /*if ( 1 == command . get_Le ( ) )
    {
        the_response = Octet_String ( dataTemplate , 1 ) ;
    }
    else if ( 2 == command . get_Le ( ) )
    {
        the_response = Octet_String ( dataTemplate , sizeof ( dataTemplate ) ) ;
    }
    else
    {*/
        try {
            the_response = Primitive_BER_TLV::generate ( where , 
                BER_TLV_Tag ( DATA_FIELD_DATA ) ,
                //BER_TLV_Length ( command . get_Le ( ) == 0 ? the_data_object . length( ) : ( command . get_Le ( ) - 2 ) ) , //It would have worked with -1!
                  BER_TLV_Length ( the_data_object . length( )) ,
                the_data_object ) ;
        } catch ( ... )
        {
            throw RAPDU_Packet::SW_WRONG_LENGTH ;
        }
    //}
	
    APDU ( the_response ) . store_into_APDU_packet ( response ) ;

    return 0 ;
  } ;

//
// VERIFY
//
long PIV_Card_Application::verify_pin ( const CAPDU_Packet & command, RAPDU_Packet & response )
{
    Octet_String the_presented_PIN ;

    if( the_security_system_pointer -> contactless_interface ( ) )
        throw RAPDU_Packet::SW_FUNCTION_NOT_SUPPORTED ;

    if ( 0x00 != command . get_P1 ( ) )
        throw RAPDU_Packet::SW_INCORRECT_P1_P2 ;
    
    Key_Reference the_key_reference ( 0 );

	try {
		the_key_reference = Key_Reference ( command . get_P2 ( ) );
	} catch ( ... )
	{
		throw RAPDU_Packet::SW_REFERENCED_DATA_NOT_FOUND ;
	}

    // The only PIN that can be verified by the PIV card application
    if ( Key_Reference ( CARDHOLDER_PIV_CARD_APPLICATION_PIN ) != the_key_reference )
        throw RAPDU_Packet::SW_REFERENCED_DATA_NOT_FOUND ;

	//if Lc=0x00 and the command data field is empty, the command can be used to 
	//retrieve the number of further retries
	if (( 0x00 == command.get_P3()) && (command.get_APDU_length() == 5))
	{
		the_presented_PIN = Octet_String(0);
		the_security_system_pointer 
        -> verify_pin ( the_key_reference , the_presented_PIN ) ;
	}
	else if ( MANDATORY_PIN_LENGTH != command . get_P3 ( ) )
        throw RAPDU_Packet::SW_WRONG_PARAMETERS_IN_DATA_FIELD ;

    try {
        the_presented_PIN  = Octet_String ( command . payload ( ) , MANDATORY_PIN_LENGTH ) ;
    } catch ( ... )
    {
        throw RAPDU_Packet::SW_WRONG_PARAMETERS_IN_DATA_FIELD ;
    }

	try {
		the_security_system_pointer 
			-> verify_pin ( the_key_reference , the_presented_PIN ) ;
	}
	catch ( RAPDU_Packet::Status_Word sw )
	{
		the_PIN_is_verified = false;
		throw sw;
	}

    the_PIN_verification_counter = 1;
	the_PIN_is_verified = true;

    return 0 ;
} ;

//
// CHANGE REFERENCE DATA
//
long PIV_Card_Application::change_reference_data ( const CAPDU_Packet & command, RAPDU_Packet & response )
{
    if( the_security_system_pointer -> contactless_interface ( ) )
        throw RAPDU_Packet::SW_FUNCTION_NOT_SUPPORTED ;

    if ( 0x00 != command . get_P1 ( ) )
        throw RAPDU_Packet::SW_INCORRECT_P1_P2 ;

    // Command data field shall contain the current PIN value and the new PIN value
    if ( ( MANDATORY_PIN_LENGTH + MANDATORY_PIN_LENGTH ) != command . get_P3 ( ) )
        throw RAPDU_Packet::SW_WRONG_PARAMETERS_IN_DATA_FIELD ;

	Key_Reference the_key_reference ( 0 );

	try {
		the_key_reference = Key_Reference ( command . get_P2 ( ) );
	} catch ( ... )
	{
		throw RAPDU_Packet::SW_REFERENCED_DATA_NOT_FOUND ;
	}

    // Only one PIN can be changed by the PIV card application
    if ( Key_Reference ( CARDHOLDER_PIV_CARD_APPLICATION_PIN ) != the_key_reference )
        throw RAPDU_Packet::SW_REFERENCED_DATA_NOT_FOUND ;

    Octet_String the_presented_PIN ( command . payload ( ) , MANDATORY_PIN_LENGTH ) ;

    Octet_String the_new_PIN ( command . payload ( MANDATORY_PIN_LENGTH ) , MANDATORY_PIN_LENGTH ) ;

    the_security_system_pointer 
        -> change_pin ( the_key_reference , the_presented_PIN , the_new_PIN ) ;

    // Changing the PIN counts as a PIN_ALWAYS
    the_PIN_verification_counter = 1;
	the_PIN_is_verified = true;

    return 0 ;
} ;

//
// RESET RETRY COUNTER
//
long PIV_Card_Application::reset_retry_counter ( const CAPDU_Packet & command, RAPDU_Packet & response )
{
    if( the_security_system_pointer -> contactless_interface ( ) )
        throw RAPDU_Packet::SW_FUNCTION_NOT_SUPPORTED ;

    if ( 0x00 != command . get_P1 ( ) )
        throw RAPDU_Packet::SW_INCORRECT_P1_P2 ;
    
	if ( CARDHOLDER_PIV_CARD_APPLICATION_PIN != command . get_P2 ( ) &&
		 CARDHOLDER_PIV_CARD_APPLICATION_PUK != command . get_P2 ( ) )
		 throw RAPDU_Packet::SW_REFERENCED_DATA_NOT_FOUND;

    Key_Reference the_key_reference ( command . get_P2 ( ) ) ;

    // The only PIN that can be unblocked by the PIV card application
    if ( Key_Reference ( CARDHOLDER_PIV_CARD_APPLICATION_PIN ) != the_key_reference )
        throw RAPDU_Packet::SW_REFERENCED_DATA_NOT_FOUND ;

    // Command data field shall contain unblocking PIN value
    if ( MANDATORY_PIN_LENGTH + MANDATORY_PIN_LENGTH != command . get_P3 ( ))
        throw RAPDU_Packet::SW_WRONG_PARAMETERS_IN_DATA_FIELD ;

    Octet_String the_unblock_PIN ( command . payload ( ) , MANDATORY_PIN_LENGTH ) ;

    Octet_String the_new_PIN ( command . payload ( MANDATORY_PIN_LENGTH ) , MANDATORY_PIN_LENGTH ) ;

    the_security_system_pointer 
        -> unblock_pin ( the_key_reference , the_unblock_PIN, the_new_PIN  ) ;

    return 0 ;
} ;


//
// GENERAL AUTHENTICATE
//
long PIV_Card_Application::general_authenticate  ( const CAPDU_Packet & command, RAPDU_Packet & response )
{
    BER_TLV_Sequence * the_payload_tlvs = NULL ;
    Primitive_BER_TLV * the_data_field = NULL ;
    Octet_String the_application_response;
    Primitive_BER_TLV_Pointer the_challenge ;
    Primitive_BER_TLV_Pointer the_witness ;
    Primitive_BER_TLV_Pointer the_response ;
	const Octet P1 = command . get_P1 ( ) ;
    const Octet P2 = command . get_P2 ( ) ;

	Octet_String the_command_payload ( command . payload ( ) , 
                                       command . get_P3 ( ) ) ;

#if !defined(T0)
	// GENERAL AUTHENTICATE must be a Case 4 command
	if ( ( command . get_P3 ( ) + 5 ) != ( command . get_APDU_length ( ) - 1 ) )
		throw RAPDU_Packet::SW_WRONG_PARAMETERS_IN_DATA_FIELD ;
#endif

    if ( CAPDU_Packet::CLAINS_INVALID == the_chained_command )
    {
        if ( DYN_TEMPLATE != the_command_payload . get_octet( 0 ) )
            throw RAPDU_Packet::SW_CONDITIONS_OF_USE_NOT_SATISFIED ;

        try {
            the_data_field = new Primitive_BER_TLV( the_command_payload );

            the_payload_tlvs = new BER_TLV_Sequence ( the_command_payload ,
                                                      the_command_payload . start ( ) + 
                                                      the_data_field->tag ( ) . length ( )  + 
                                                      the_data_field->length ( ) . length ( ) ,
                                                      the_command_payload . stop ( ) ) ;
        } catch ( ... )
        {
            delete the_data_field ;
            delete the_payload_tlvs ;
            throw RAPDU_Packet::SW_WRONG_PARAMETERS_IN_DATA_FIELD ;
        }
    }
    else
    {
        the_chained_command = CAPDU_Packet::CLAINS_INVALID ;

		try {
			the_chained_data = Octet_String::concat( the_chained_data, the_command_payload );
		} catch ( ... )
		{
			throw RAPDU_Packet::SW_NOT_ENOUGH_SPACE;
		}

        if ( DYN_TEMPLATE != the_chained_data . get_octet( 0 ) )
            throw RAPDU_Packet::SW_CONDITIONS_OF_USE_NOT_SATISFIED ;

        try {
            the_data_field = new Primitive_BER_TLV( the_chained_data );

            the_payload_tlvs = new BER_TLV_Sequence ( the_chained_data ,
                                                      the_chained_data . start ( ) + 
                                                      the_data_field->tag ( ) . length ( )  + 
                                                      the_data_field->length ( ) . length ( ) ,
                                                      the_chained_data . stop ( ) ) ;
        } catch ( ... )
        {
            delete the_data_field ;
            delete the_payload_tlvs ;
            throw RAPDU_Packet::SW_WRONG_PARAMETERS_IN_DATA_FIELD ;
        }
    }

    try {
        the_challenge = the_payload_tlvs -> find ( DYN_CHALLENGE ) ;
    } 
    catch ( ... )
    {
    }

    // GET CHALLENGE
    if ( Primitive_BER_TLV_Pointer::null( ) != the_challenge &&
        0 == the_challenge->length().value())
    {
		//Check the key ref and algorithm more than if == 0x00.
        if ( 0x00 == command . get_P2 ( ) && 0x00 == command . get_P1 ( ) )
            throw RAPDU_Packet::SW_INCORRECT_P1_P2;
		
		switch ( P2 )
		{
			//asymmetric keys
			case 0x9A:
			case 0x9C:
			case 0x9D:
				if(((P1 >= 0x00) && (P1 <= 0x04)) || ((P1 >= 0x08) && (P1 <= 0x0D))) //one of the defined symetric algs
					throw RAPDU_Packet::SW_INCORRECT_P1_P2;

			// symmetric key
			case 0x9B:
				if(((P1 >= 0x05) && (P1 <= 0x07)) || ((P1 >= 0x0E) && (P1 <= 0x13))) //if one of defined asymetric algs
					throw RAPDU_Packet::SW_INCORRECT_P1_P2;

			// both types
			case 0x9E:
				break;

			// unknown key type
			default:
				throw RAPDU_Packet::SW_INCORRECT_P1_P2;
		};

        Octet_String & the_challenge = * new Octet_String ( MANDATORY_CHALLENGE_LENGTH ) ;
        the_security_system_pointer -> generate_cryptogram ( the_challenge , MANDATORY_CHALLENGE_LENGTH ) ;

        the_challenge_pointer = 
            Octet_String_Pointer::object_pointer_to_indexed_pointer ( & the_challenge ) ;

        // Save the challenge for one additional (possibly chained) command
        the_challenge_preservation_counter = 1;

        the_application_response = Octet_String ( get_challenge_response_octets, sizeof ( get_challenge_response_octets ) ) ;
		Octet_String::cpy ( the_application_response , 4 , the_challenge, 0, MANDATORY_CHALLENGE_LENGTH );
		
	/*	if ( command . get_Le ( ) != 0 )
			the_application_response = Octet_String ( the_application_response, 0, command . get_Le ( ) ) ; */

        APDU ( the_application_response ) . store_into_APDU_packet ( response ) ;

        delete the_data_field ;
        delete the_payload_tlvs ;

        return 0 ;
    }

    // Common processing for all PIV card application authentication protocols
    Algorithm_Identifier the_algorithm_identifier ( command . get_P1 ( ) ) ;

    Key_Reference the_key_reference ( command . get_P2 ( ) ) ;

    // Key reference 9A shall not be used on the contactless interface
	if( the_security_system_pointer -> contactless_interface ( ) &&
                                          ( Key_Reference ( PIV_CARD_APPLICATION_VALIDATION ) == the_key_reference ) )
        throw RAPDU_Packet::SW_CONDITIONS_OF_USE_NOT_SATISFIED ;

	// Check that the cyptographic algorithm matches the key reference
	switch ( command . get_P2 ( ) )
    {
		case PIV_CARD_APPLICATION_KEY_MANAGEMENT:
        case PIV_CARD_AUTHENTICATION_KEY:
		case PIV_CARD_APPLICATION_VALIDATION:
		case PIV_CARD_APPLICATION_DIGITAL_SIGNATURE:
			if( command . get_P1 ( ) != RSA_1024 )
			{
				delete the_data_field ;
				delete the_payload_tlvs ;
				throw RAPDU_Packet::SW_INCORRECT_P1_P2 ;
			}
			else
				break;

		case PIV_CARD_APPLICATION_ADMINISTRATION:
			if( command . get_P1 ( ) != DEFAULT_ALGORITHM &&
				command . get_P1 ( ) != TRIPLE_DES_ECB_192 )
			{
				delete the_data_field ;
				delete the_payload_tlvs ;
				throw RAPDU_Packet::SW_INCORRECT_P1_P2 ;
			}
			else
				break;

		default:
				delete the_data_field ;
				delete the_payload_tlvs ;
				throw RAPDU_Packet::SW_INCORRECT_P1_P2 ;
	}

    if( Algorithm_Identifier ( DEFAULT_ALGORITHM ) == the_algorithm_identifier )
        the_algorithm_identifier = Algorithm_Identifier ( DEFAULT_ALGORITHM_IDENTIFIER ) ;

    the_security_system_pointer->the_security_environment.the_algorithm_identifier = the_algorithm_identifier ;
    the_security_system_pointer->the_security_environment.the_key_reference = the_key_reference ;

    try {
        the_response = the_payload_tlvs -> find ( DYN_RESPONSE ) ;
    } catch ( ... )
    {
    }

    try {
        the_witness = the_payload_tlvs -> find ( DYN_WITNESS ) ;
    } catch ( ... )
    {
    }

    delete the_data_field ;
    delete the_payload_tlvs ;

    if ( BER_TLV_Pointer::null ( ) == the_response  &&
         BER_TLV_Pointer::null ( ) == the_challenge &&
         BER_TLV_Pointer::null ( ) == the_witness )
    {
        throw RAPDU_Packet::SW_WRONG_PARAMETERS_IN_DATA_FIELD ;
    }

    // INTERNAL AUTHENTICATE
    if ( BER_TLV_Pointer::null ( ) != the_challenge     &&
         0 < the_challenge -> length ( ) . value ( )    &&
         BER_TLV_Pointer::null ( ) != the_response      &&
         0 == the_response -> length ( ) . value ( ) )
    {
        Octet_String the_computed_cryptogram ( MAXIMUM_CRYPTOGRAM_LENGTH ) ;
		BYTE return_size = 0;

	    // INTERNAL AUTHENTICATE with '9A' requires PIN, with '9C' requires PIN_Always condition
	    if (( Key_Reference ( PIV_CARD_APPLICATION_VALIDATION ) == the_key_reference && !the_PIN_is_verified) ||
			( Key_Reference ( PIV_CARD_APPLICATION_DIGITAL_SIGNATURE ) == the_key_reference && PIN_Always == 0 ))
			throw RAPDU_Packet::SW_SECURITY_STATUS_NOT_SATISFIED ;
        
        try {
            the_security_system_pointer
                -> encrypt ( the_challenge -> values ( ) ,
                             the_computed_cryptogram ) ;
        } catch ( ... )
        {
            throw RAPDU_Packet::SW_CONDITIONS_OF_USE_NOT_SATISFIED ;
        }

		/*if( command . get_Le() != 0x00 )
		{
			return_size = command . get_Le() ;
			if( Algorithm_Identifier (RSA_1024) == the_algorithm_identifier )
				the_application_response = Octet_String ( internal_authenticate_response_RSA_1024, return_size ) ;
			else
				the_application_response = Octet_String ( internal_authenticate_response, return_size ) ;
		}
		else
		{*/
			if( Algorithm_Identifier (RSA_1024) == the_algorithm_identifier )
			{
				return_size = RSA_ENCRYPTED_CHALLENGE_LENGTH + 6 ;
				the_application_response = Octet_String ( internal_authenticate_response_RSA_1024, return_size ) ;
				Octet_String::cpy ( the_application_response, 6 , the_computed_cryptogram , 0, return_size - 6 ) ;
			}
			else
			{
				return_size = MANDATORY_CHALLENGE_LENGTH + 4 ;
				the_application_response = Octet_String ( internal_authenticate_response, return_size ) ;
				Octet_String::cpy ( the_application_response, 4 , the_computed_cryptogram , 0, return_size - 4 ) ;
			}
		//}

        APDU ( the_application_response ) . store_into_APDU_packet ( response ) ;

        return 0;
    }

    // EXTERNAL AUTHENTICATE
    else if ( BER_TLV_Pointer::null ( ) != the_response && 
              0 < the_response -> length ( ) . value ( ) ) 
    {
        //Do not allow External Authenticate for RSA keys as of now
        if( Key_Reference ( PIV_CARD_APPLICATION_VALIDATION ) == the_key_reference ||
            Key_Reference ( PIV_CARD_APPLICATION_DIGITAL_SIGNATURE ) == the_key_reference ||
            Key_Reference ( PIV_CARD_APPLICATION_KEY_MANAGEMENT ) == the_key_reference)
            throw RAPDU_Packet::SW_CONDITIONS_OF_USE_NOT_SATISFIED;

        the_security_system_pointer
            -> set_authentication ( the_key_reference , false ) ;

        if ( Octet_String_Pointer::null ( ) == the_challenge_pointer )
            throw RAPDU_Packet::SW_CONDITIONS_OF_USE_NOT_SATISFIED ;

        Octet_String the_plaintext ( MANDATORY_CHALLENGE_LENGTH ) ;

        try {
            the_security_system_pointer 
                -> decrypt( the_response -> values ( ),
                            the_plaintext ) ; 
        } catch ( ... )
        {
            throw RAPDU_Packet::SW_CONDITIONS_OF_USE_NOT_SATISFIED ;
        }

        bool authenticated = 
            0 == Octet_String::cmp ( * the_challenge_pointer , 
                                       the_plaintext , 
                                       MANDATORY_CHALLENGE_LENGTH ) ;

        the_security_system_pointer
            -> set_authentication ( the_key_reference , authenticated ) ;

        if ( ! authenticated )
            throw RAPDU_Packet::SW_VERIFICATION_FAILED ;
        else
		{
			external_authentication_passed = true ;
            return 0;
		}
    }
    // MUTUAL AUTHENTICATE - Start
    else if ( BER_TLV_Pointer::null ( ) != the_witness && 
              0 == the_witness -> length ( ) . value ( ) ) 
    {
        //Do not allow Mutual Authenticate for RSA keys as of now
        if( Key_Reference ( PIV_CARD_APPLICATION_VALIDATION ) == the_key_reference ||
            Key_Reference ( PIV_CARD_APPLICATION_DIGITAL_SIGNATURE ) == the_key_reference ||
            Key_Reference ( PIV_CARD_APPLICATION_KEY_MANAGEMENT ) == the_key_reference)
            throw RAPDU_Packet::SW_CONDITIONS_OF_USE_NOT_SATISFIED;

        Octet_String & the_challenge = * new Octet_String ( MANDATORY_CHALLENGE_LENGTH ) ;
        the_security_system_pointer -> generate_cryptogram ( the_challenge , MANDATORY_CHALLENGE_LENGTH ) ;

        the_challenge_pointer = 
            Octet_String_Pointer::object_pointer_to_indexed_pointer ( & the_challenge ) ;

        // Save the challenge for one additional (possibly chained) command
        the_challenge_preservation_counter = 1;

        Octet_String the_computed_cryptogram ( MAXIMUM_CRYPTOGRAM_LENGTH ) ;

        try {
            the_security_system_pointer
                -> encrypt ( the_challenge  ,
                             the_computed_cryptogram ) ; 
        } catch ( ... )
        {
            throw RAPDU_Packet::SW_CONDITIONS_OF_USE_NOT_SATISFIED ;
        }

		BYTE return_size = 0;

		/*if( command . get_Le() != 0x00 )
		{
			return_size = command . get_Le() ;
			if( Algorithm_Identifier (RSA_1024) == the_algorithm_identifier )
				the_application_response = Octet_String ( mutual_authenticate_response_RSA_1024, return_size ) ;
			else
				the_application_response = Octet_String ( mutual_authenticate_response, return_size ) ;
		}
		else
		{*/
			if( Algorithm_Identifier (RSA_1024) == the_algorithm_identifier )
			{
				return_size = RSA_ENCRYPTED_CHALLENGE_LENGTH + 6 ;
				the_application_response = Octet_String ( mutual_authenticate_response_RSA_1024, return_size ) ;
				Octet_String::cpy ( the_application_response, 6 , the_computed_cryptogram , 0, return_size - 6 ) ;
			}
			else
			{
				return_size = MANDATORY_CHALLENGE_LENGTH + 4 ;
				the_application_response = Octet_String ( mutual_authenticate_response, return_size ) ;
				Octet_String::cpy ( the_application_response, 4 , the_computed_cryptogram , 0, return_size - 4 ) ;
			}
		//}

        APDU ( the_application_response ) . store_into_APDU_packet ( response ) ;

        return 0;
    }
    // MUTUAL AUTHENTICATE - Finish
    else if ( BER_TLV_Pointer::null ( ) != the_witness && 
              0 != the_witness -> length ( ) . value ( ) &&
              BER_TLV_Pointer::null ( ) != the_challenge && 
              0 != the_challenge -> length ( ) . value ( ) ) 
    {
        //Do not allow Mutual Authenticate for RSA keys as of now
        if( Key_Reference ( PIV_CARD_APPLICATION_VALIDATION ) == the_key_reference ||
            Key_Reference ( PIV_CARD_APPLICATION_DIGITAL_SIGNATURE ) == the_key_reference ||
            Key_Reference ( PIV_CARD_APPLICATION_KEY_MANAGEMENT ) == the_key_reference)
            throw RAPDU_Packet::SW_CONDITIONS_OF_USE_NOT_SATISFIED;

        /* Check of the Terminal's Decryption of the Card Witness */
        the_security_system_pointer
            -> set_authentication ( the_key_reference , false ) ;

        if ( Octet_String_Pointer::null ( ) == the_challenge_pointer )
            throw RAPDU_Packet::SW_CONDITIONS_OF_USE_NOT_SATISFIED ;

        bool authenticated = 
            0 == Octet_String::cmp (   * the_challenge_pointer , 
                                       the_witness->values( ) , 
                                       MANDATORY_CHALLENGE_LENGTH ) ;

        the_security_system_pointer
            -> set_authentication ( the_key_reference , authenticated ) ;

        if ( ! authenticated )
        {
            throw RAPDU_Packet::SW_VERIFICATION_FAILED ;
        }
		else
		{
			mutual_authentication_passed = true ;
		}

        /* Encryption of the Terminal Nonce */
        Octet_String the_computed_cryptogram ( MAXIMUM_CRYPTOGRAM_LENGTH ) ;
        
        try {
            the_security_system_pointer
                -> encrypt ( the_challenge -> values ( ) ,
                             the_computed_cryptogram ) ; 
        } catch ( ... )
        {
            throw RAPDU_Packet::SW_CONDITIONS_OF_USE_NOT_SATISFIED ;
        }

		BYTE return_size = 0;

		/*if( command . get_Le() != 0x00 )
		{
			return_size = command . get_Le() ;
			if( Algorithm_Identifier (RSA_1024) == the_algorithm_identifier )
				the_application_response = Octet_String ( mutual_authenticate_response_RSA_1024, return_size ) ;
			else
				the_application_response = Octet_String ( mutual_authenticate_response, return_size ) ;
		}
		else
		{*/
			if( Algorithm_Identifier (RSA_1024) == the_algorithm_identifier )
			{
				return_size = RSA_ENCRYPTED_CHALLENGE_LENGTH + 4 ;
				the_application_response = Octet_String ( mutual_authenticate_response_RSA_1024, return_size ) ;
			}
			else
			{
				return_size = MANDATORY_CHALLENGE_LENGTH + 4 ;
				the_application_response = Octet_String ( mutual_authenticate_response, return_size ) ;
			}
		//}

  		if( return_size > 4 )
			Octet_String::cpy ( the_application_response, 4 , the_computed_cryptogram , 0, return_size - 4 ) ;

        APDU ( the_application_response ) . store_into_APDU_packet ( response ) ;

        return 0;

    }
    // Malformed GENERAL AUTHENTICATE Command
    else
    {
        throw RAPDU_Packet::SW_CONDITIONS_OF_USE_NOT_SATISFIED ;
    }
    return 0 ;
} ;

//
// PUT DATA
//
long PIV_Card_Application::put_data ( const CAPDU_Packet & command, RAPDU_Packet & response )
{
    BER_TLV_Sequence * the_payload_tlvs = NULL ;
    Primitive_BER_TLV_Pointer the_tag_list ;
    Primitive_BER_TLV_Pointer the_data_object ;

    if( the_security_system_pointer -> contactless_interface ( ) )
        throw RAPDU_Packet::SW_FUNCTION_NOT_SUPPORTED ;

    if ( 0x3F != command . get_P1 ( ) ||  0xFF != command . get_P2 ( ))
        throw RAPDU_Packet::SW_INCORRECT_P1_P2 ;

    Octet_String the_command_payload ( command . payload ( ) , 
                                       command . get_P3 ( ) ) ;

    if ( CAPDU_Packet::CLAINS_INVALID == the_chained_command )
    {
        try {
            the_payload_tlvs = new BER_TLV_Sequence ( the_command_payload ,
                                                      the_command_payload . start ( ) ,
                                                      the_command_payload . stop ( ) ) ;
        } catch ( ... )
        {
            delete the_payload_tlvs ;
            throw RAPDU_Packet::SW_WRONG_PARAMETERS_IN_DATA_FIELD ;
        }
    }
    else
    {
        the_chained_command = CAPDU_Packet::CLAINS_INVALID ;
		try {
			the_chained_data = Octet_String::concat(the_chained_data, the_command_payload );
		} catch ( ... )
		{
			throw RAPDU_Packet::SW_NOT_ENOUGH_SPACE;
		}

        try {
            the_payload_tlvs = new BER_TLV_Sequence ( the_chained_data ,
                                                      the_chained_data . start ( ) ,
                                                      the_chained_data . stop ( ) ) ;
        } catch ( ... )
        {
            delete the_payload_tlvs ;
            throw RAPDU_Packet::SW_WRONG_PARAMETERS_IN_DATA_FIELD ;
        }
    }

    try {
        the_tag_list =
                the_payload_tlvs -> find ( DATA_FIELD_TAG_LIST ) ;
    } catch ( ... )
    {
        delete the_payload_tlvs ;
        throw RAPDU_Packet::SW_WRONG_PARAMETERS_IN_DATA_FIELD ;
    }

    if ( DATA_FIELD_TAG_LIST != the_tag_list -> tag ( ) . get_octet ( 0 ) ||
         3 != the_tag_list -> length ( ) . get_octet ( 0 ) )
    {
        delete the_payload_tlvs ;
        throw RAPDU_Packet::SW_WRONG_PARAMETERS_IN_DATA_FIELD ;
    }

    BER_TLV_Tag the_tag ( the_tag_list -> values( ) );

	the_file_system_pointer -> verify_data_object_authorization ( the_tag , ACR_AM_DO_PUT_DATA ) ;

    try {
        the_data_object =
                the_payload_tlvs -> find ( DATA_FIELD_PRIMITIVE_BER_TLV ) ;
    } catch ( ... )
    {
        delete the_payload_tlvs ;
        throw RAPDU_Packet::SW_WRONG_PARAMETERS_IN_DATA_FIELD ;
    }
	
    the_file_system_pointer -> put_data ( the_tag, the_data_object-> values ( ) ) ;

    delete the_payload_tlvs ;

    return 0;
  } ;

//
// GENERATE ASYMMETRIC KEY PAIR
//
long PIV_Card_Application::generate_asymmetric_key_pair ( const CAPDU_Packet & command, RAPDU_Packet & response )
{
    BER_TLV_Sequence * the_payload_tlvs = NULL ;
    Primitive_BER_TLV * the_data_field = NULL ;
    Primitive_BER_TLV_Pointer the_mechanism ;
    Octet mechanism ;
    unsigned int exponent = 0 ;
    Primitive_BER_TLV_Pointer the_parameters ;
    RSA_Key_Pointer the_new_RSA_key_pointer ;
    ECC_Key_Pointer the_new_ECC_key_pointer ;

    if( the_security_system_pointer -> contactless_interface ( ) )
        throw RAPDU_Packet::SW_FUNCTION_NOT_SUPPORTED ;

#if !defined(T0)
	// GENERATE ASYMMETRIC KEY PAIR must be a Case 4 command
	if ( ( command . get_P3 ( ) + 5 ) != ( command . get_APDU_length ( ) - 1 ) )
		throw RAPDU_Packet::SW_WRONG_PARAMETERS_IN_DATA_FIELD ;
#endif

    if ( 0x00 != command . get_P1 ( ) )
        throw RAPDU_Packet::SW_INCORRECT_P1_P2 ;

	//Generate asymetric key requires only external autehnticate in 800-73-1
	//if ( false == mutual_authentication_passed )
	if ( false == external_authentication_passed )
		throw RAPDU_Packet::SW_SECURITY_STATUS_NOT_SATISFIED ;

    Octet_String the_command_payload ( command . payload ( ) , 
                                       command . get_P3 ( ) ) ;

    if ( CAPDU_Packet::CLAINS_INVALID == the_chained_command )
    {
		if ( GAK_TEMPLATE != the_command_payload . get_octet( 0 ) )
            throw RAPDU_Packet::SW_CONDITIONS_OF_USE_NOT_SATISFIED ;

        try {
            the_data_field = new Primitive_BER_TLV( the_command_payload );

            the_payload_tlvs = new BER_TLV_Sequence ( the_command_payload ,
                                                      the_command_payload . start ( ) + 
                                                      the_data_field->tag ( ) . length ( ) + 
                                                      the_data_field->length ( ) . length ( ) ,
                                                      the_command_payload . stop ( ) ) ;
        } catch ( ... )
        {
            delete the_data_field ;
            delete the_payload_tlvs ;
            throw RAPDU_Packet::SW_WRONG_PARAMETERS_IN_DATA_FIELD ;
        }
    }
    else
    {
        the_chained_command = CAPDU_Packet::CLAINS_INVALID ;

		try {
			the_chained_data = Octet_String::concat( the_chained_data, the_command_payload );
		} catch ( ... )
		{
			throw RAPDU_Packet::SW_NOT_ENOUGH_SPACE;
		}

        if ( GAK_TEMPLATE != the_chained_data . get_octet( 0 ) )
            throw RAPDU_Packet::SW_CONDITIONS_OF_USE_NOT_SATISFIED ;

        try {
            the_data_field = new Primitive_BER_TLV( the_chained_data );

            the_payload_tlvs = new BER_TLV_Sequence ( the_chained_data ,
                                                      the_chained_data . start ( ) + 
                                                      the_data_field->tag ( ) . length ( )  + 
                                                      the_data_field->length ( ) . length ( ) ,
                                                      the_chained_data . stop ( ) ) ;
        } catch ( ... )
        {
            delete the_data_field ;
            delete the_payload_tlvs ;
            throw RAPDU_Packet::SW_WRONG_PARAMETERS_IN_DATA_FIELD ;
        }
    }

    Octet the_generation_control = command . get_P1 ( ) ;
    Octet the_new_key_reference  = command . get_P2 ( ) ;

    try {
        the_mechanism = the_payload_tlvs -> find ( GAK_MECHANISM ) ;
    } catch ( ... )
    {
        delete the_data_field ;
        delete the_payload_tlvs ;
        throw RAPDU_Packet::SW_WRONG_PARAMETERS_IN_DATA_FIELD ;
    }
	
	// Check that a valid cryptographic algorithm is used
	if ( the_mechanism -> values( )[0] != RSA_1024 &&
		 the_mechanism -> values( )[0] != DEFAULT_ALGORITHM &&
		 the_mechanism -> values( )[0] != TRIPLE_DES_ECB_192 )
	{
		delete the_data_field ;
		delete the_payload_tlvs ;
		throw RAPDU_Packet::SW_WRONG_PARAMETERS_IN_DATA_FIELD ;
	}

	// Check that the cryptographic algorithm matches the key reference
	switch ( the_new_key_reference )
    {
		case PIV_CARD_APPLICATION_VALIDATION:
		case PIV_CARD_APPLICATION_DIGITAL_SIGNATURE:
		case PIV_CARD_APPLICATION_KEY_MANAGEMENT:
			if( the_mechanism -> values( )[0] != RSA_1024 )
			{
				delete the_data_field ;
				delete the_payload_tlvs ;
				throw RAPDU_Packet::SW_INCORRECT_P1_P2 ;
			}
			else
				break;
		case PIV_CARD_APPLICATION_ADMINISTRATION:
			if( the_mechanism -> values( )[0] != DEFAULT_ALGORITHM &&
				the_mechanism -> values( )[0] != TRIPLE_DES_ECB_192 )
			{
				delete the_data_field ;
				delete the_payload_tlvs ;
				throw RAPDU_Packet::SW_INCORRECT_P1_P2 ;
			}
			else
				break;
		default:
			delete the_data_field ;
			delete the_payload_tlvs ;
			throw RAPDU_Packet::SW_INCORRECT_P1_P2 ;
	}

    if ( the_mechanism == Primitive_BER_TLV_Pointer::null( ) || 
         the_mechanism -> length( ) . value( ) != 1 )
    {
        delete the_data_field ;
        delete the_payload_tlvs ;
        throw RAPDU_Packet::SW_WRONG_PARAMETERS_IN_DATA_FIELD ;
    }
    mechanism = the_mechanism->values( )[0] ;
    if( mechanism == RSA_1024 || mechanism == RSA_2048 ||mechanism == RSA_3072 )
    {
        try {
            the_parameters = the_payload_tlvs -> find ( GAK_PARAMETERS ) ;
        } catch ( ... )
        {
            exponent = 65537 ;
        };

        if ( exponent == 0 )
        {
            if ( the_parameters -> length( ) . value( ) != 3 )
            {
                delete the_data_field ;
                delete the_payload_tlvs ;
                throw RAPDU_Packet::SW_WRONG_PARAMETERS_IN_DATA_FIELD ;
            } 
            else
            {
                // A provided public exponent must be a 3-byte integer
                exponent = the_parameters -> values( ) . get_u24( 0 ) ;
            }
        }

        delete the_data_field ;
        delete the_payload_tlvs ;

        try {
            the_new_RSA_key_pointer 
                = the_security_system_pointer->Generate_RSA_Key_Pair ( key_size_in_bits[ mechanism ] , exponent ) ;
        } catch ( ... )
        {
            throw RAPDU_Packet::SW_WRONG_PARAMETERS_IN_DATA_FIELD ;
        }

        RSA_Key_Entry * the_new_RSA_key_entry =
            new RSA_Key_Entry ( the_new_RSA_key_pointer ) ;

        the_security_system_pointer
            -> the_key_store . put_key_entry_pointer 
                ( the_new_key_reference , 
                    RSA_Key_Entry_Pointer::object_pointer_to_indexed_pointer ( the_new_RSA_key_entry ) ) ;

        // Return the public key in the public key template
        Octet_String public_key_template ( MAXIMUM_RESPONSE ) ;
        Octet_String where ( public_key_template ) ;

        Primitive_BER_TLV exp = Primitive_BER_TLV::generate ( where , 
            BER_TLV_Tag ( PK_RSA_PUBLIC_EXPONENT ) ,
            BER_TLV_Length ( the_new_RSA_key_entry -> key_pointer -> e . length( ) ) ,
            the_new_RSA_key_entry -> key_pointer -> e ) ;

        Primitive_BER_TLV mod = Primitive_BER_TLV::generate ( where , 
            BER_TLV_Tag ( PK_RSA_MODULUS ) ,
            BER_TLV_Length ( the_new_RSA_key_entry -> key_pointer -> m . length( ) ) ,
            the_new_RSA_key_entry -> key_pointer -> m ) ;

		Constructed_BER_TLV the_public_key = Constructed_BER_TLV::generate ( public_key_template ,
			BER_TLV_Tag ( Octet_String ( publicKeyTemplate , sizeof( publicKeyTemplate ) ) ) ,
			BER_TLV_Length ( exp.total_length( ) +  mod.total_length ( ) ) ) ;

#if !defined(T0)
		if ( command . get_Le( ) != 0x00 )
		{ 
			Octet_String the_truncated_key = the_public_key . substring( 0,  command . get_Le( ) );
			APDU ( the_truncated_key ) . store_into_APDU_packet ( response ) ;
		}
		else
		{
			APDU ( the_public_key ) . store_into_APDU_packet ( response ) ;
		}
#else
        APDU ( the_public_key ) . store_into_APDU_packet ( response ) ;
#endif

    }
    else if( mechanism == ECC_P_224 || mechanism == ECC_K_233 || mechanism == ECC_B_233 ||
             mechanism == ECC_P_256 || mechanism == ECC_K_283 || mechanism == ECC_B_283 )
    {
        delete the_data_field ;
        delete the_payload_tlvs ;

        try {
            the_new_ECC_key_pointer 
                = the_security_system_pointer->Generate_ECC_Key_Pair ( mechanism ) ;
        } catch ( ... )
        {
            throw RAPDU_Packet::SW_WRONG_PARAMETERS_IN_DATA_FIELD ;
        }
        ECC_Key_Entry * the_new_ECC_key_entry =
            new ECC_Key_Entry ( the_new_ECC_key_pointer ) ;

        the_security_system_pointer
            -> the_key_store . put_key_entry_pointer 
                ( the_new_key_reference , 
                    ECC_Key_Entry_Pointer::object_pointer_to_indexed_pointer ( the_new_ECC_key_entry ) ) ;

        // Return the public key in the public key template
        Octet_String public_key_template ( MAXIMUM_RESPONSE ) ;
        Octet_String where ( public_key_template ) ;

        Primitive_BER_TLV exp = Primitive_BER_TLV::generate ( where , 
            BER_TLV_Tag ( PK_ECC_POINT_X ) ,
            BER_TLV_Length ( the_new_ECC_key_entry -> key_pointer -> x . length( ) ) ,
            the_new_ECC_key_entry -> key_pointer -> x ) ;

        Primitive_BER_TLV mod = Primitive_BER_TLV::generate ( where , 
            BER_TLV_Tag ( PK_ECC_POINT_Y ) ,
            BER_TLV_Length ( the_new_ECC_key_entry -> key_pointer -> y . length( ) ) ,
            the_new_ECC_key_entry -> key_pointer -> y ) ;
		
		// Zero out key entry now that it's no longer needed and delete
		delete the_new_ECC_key_entry ;

		Key_Entry_Pointer the_dummy_key_entry_pointer = 
			the_security_system_pointer -> the_key_store . get_key_entry_pointer ( DUMMY_KEY_REFERENCE ) ;

		the_security_system_pointer -> the_key_store 
			. put_key_entry_pointer ( the_new_key_reference, the_dummy_key_entry_pointer ) ;

        Constructed_BER_TLV the_public_key = Constructed_BER_TLV::generate ( public_key_template ,
            BER_TLV_Tag ( Octet_String ( publicKeyTemplate , sizeof( publicKeyTemplate ) ) ) ,
            BER_TLV_Length ( exp.total_length( ) +  mod.total_length ( ) ) ) ;

		if ( command . get_Le( ) != 0x00 )
		{ 
			Octet_String the_truncated_key = the_public_key . substring( 0,  command . get_Le( ) );
			APDU ( the_truncated_key ) . store_into_APDU_packet ( response ) ;
		}
		else
		{
			APDU ( the_public_key ) . store_into_APDU_packet ( response ) ;
		}
    }
    else
    {
        throw RAPDU_Packet::SW_WRONG_PARAMETERS_IN_DATA_FIELD ;
    }

    return 0 ;
} ;

//
// Command Chaining
//
long PIV_Card_Application::command_chaining ( const CAPDU_Packet & command, RAPDU_Packet & response )
{
    const CLAINS clains = command . get_CLAINS ( );

    // Start a new chain
    if ( CAPDU_Packet::CLAINS_INVALID == the_chained_command )
    {
      the_chained_data = Octet_String( 0 ) ;
      the_chained_command = clains ;
      the_chained_P1 = command . get_P1 ( ) ;
      the_chained_P2 = command . get_P2 ( ) ;
    }

    Octet_String the_command_payload ( command . payload ( ) , 
                                       command . get_P3 ( ) ) ;
	try {
		the_chained_data = Octet_String::concat( the_chained_data , the_command_payload );
	} catch ( ... )
	{
		throw RAPDU_Packet::SW_NOT_ENOUGH_SPACE;
	}

    return 0 ;
} ;

//
// Command Dispatch
//
PIV_Card_Application::Dispatch_Table_Entry PIV_Card_Application::dispatch_table [ ] =
{
    { CAPDU_Packet::CLAINS_SELECT                       , select_application           } ,
    { CAPDU_Packet::CLAINS_GET_DATA                     , get_data                     } ,
    { CAPDU_Packet::CLAINS_VERIFY                       , verify_pin                   } ,
    { CAPDU_Packet::CLAINS_CHANGE_REFERENCE_DATA        , change_reference_data        } ,
    { CAPDU_Packet::CLAINS_RESET_RETRY_COUNTER          , reset_retry_counter          } ,
    { CAPDU_Packet::CLAINS_GENERAL_AUTHENTICATE         , general_authenticate         } ,
    { CAPDU_Packet::CLAINS_PUT_DATA                     , put_data                     } ,
    { CAPDU_Packet::CLAINS_GENERATE_ASYMMETRIC_KEY_PAIR , generate_asymmetric_key_pair } , 

    { CAPDU_Packet::CLAINS_PUT_DATA_CHAIN                     , command_chaining } ,
    { CAPDU_Packet::CLAINS_GENERAL_AUTHENTICATE_CHAIN         , command_chaining } ,
    { CAPDU_Packet::CLAINS_GENERATE_ASYMMETRIC_KEY_PAIR_CHAIN , command_chaining } 
} ; 


long PIV_Card_Application::exchange_APDUs ( const CAPDU_Packet & command , 
                                                  RAPDU_Packet & response )
{
    // Dispatch on the CLA and INS 
    const CLAINS clains = command . get_CLAINS ( );
    const int dispatch_table_size = sizeof ( dispatch_table ) / sizeof ( dispatch_table [ 0 ] ) ;

    // Take care of updating the chaining state first

    // Chaining for transaction processing is not supported in the PIV Card Application so
    // if we're chaining then INS, P1 and P2 have to be constant across all commands in the chain
    if ( ( CAPDU_Packet::CLAINS_INVALID != the_chained_command ) &&
         ( ! ( clains == the_chained_command || clains == ( the_chained_command & CHAIN_BIT_MASK ) ) ||
             ( the_chained_P1 != command . get_P1 ( ) ) || 
             ( the_chained_P2 != command . get_P2 ( ) ) ) )
    {
        the_chained_command = CAPDU_Packet::CLAINS_INVALID ;
        throw RAPDU_Packet::SW_LAST_COMMAND_OF_CHAIN_EXPECTED ;
    }

    for ( int i = 0  ; i < dispatch_table_size ; i ++ )
      if ( dispatch_table [ i ] . clains == clains )
        break ;

    if ( i == dispatch_table_size )
      throw RAPDU_Packet::SW_FUNCTION_NOT_SUPPORTED ;



    if ( 0 == the_challenge_preservation_counter )
    {
        the_challenge_pointer = Octet_String_Pointer::null( ) ;
    }
    // Preserve the challenge over a chained GENERAL AUTHENTICATE
    else if ( CAPDU_Packet::CLAINS_INVALID == the_chained_command )
    {
        the_challenge_preservation_counter = 0;
		PIN_Always = the_PIN_verification_counter ;
        the_PIN_verification_counter = 0;
    }

    if ( CAPDU_Packet::CLAINS_INVALID == the_chained_command )
    {
        PIN_Always = the_PIN_verification_counter ;
        the_PIN_verification_counter = 0;
    }

    return ( this ->* dispatch_table [ i ] . APDU_handler ) ( command, response ) ;
} ;

#define PIV_CARD_APPLICATION_APP_AID NIST_RID, 0x00, 0x00, 0x10, 0x00, 0x01, 0x00

static Octet the_piv_card_app_AID_octets [ ] = { PIV_CARD_APPLICATION_APP_AID } ;

PIV_Card_Application::PIV_Card_Application( File_Manager * file_manager_pointer ,
                                                  const File_System_Pointer & file_system_pointer ,
                                                  const Security_System_Pointer & security_system_pointer ,
                                                  Octet_String_Pointer & challenge_pointer )
  : Application( file_manager_pointer ,
                 file_system_pointer ,
                 security_system_pointer ,
                 challenge_pointer ) ,
    the_AID ( the_piv_card_app_AID_octets , sizeof ( the_piv_card_app_AID_octets ) ) 
  {
    const int dispatch_table_size = sizeof ( dispatch_table ) / sizeof ( dispatch_table [ 0 ] ) ;
    for ( int i = 0  ; i < dispatch_table_size ; i ++ )
      file_manager_pointer -> record_APDU_handler ( dispatch_table [ i ] . clains , this ) ;
    the_chained_command = CAPDU_Packet::CLAINS_INVALID ;
    the_challenge_preservation_counter = 0;
    the_PIN_verification_counter = 0;
	the_PIN_is_verified = false;
  } ;

PIV_CARD_APPLICATION_DLL Application * 
create_application ( File_Manager * file_manager_pointer ,
                     const File_System_Pointer & file_system_pointer ,
                     const Security_System_Pointer & security_system_pointer ,
                     Octet_String_Pointer & challenge_pointer  )
{
    return 
        new PIV_Card_Application ( file_manager_pointer ,
                                   file_system_pointer ,
                                   security_system_pointer ,
                                   challenge_pointer ) ;
} ;
