// Application Manager
//
// COPYRIGHT
// ---------
// The software was developed by the National Institute of Standards and 
// Technology (NIST), an agency of the Federal Government.  Pursuant to 
// Title 15 United States Code Section 105, works of NIST are not subject
// to copyright protection in the United States and are considered to be
// in the public domain.  As a result, a formal license is not needed to
// use the software.
// 
// The Government Smart Card Reference Implementation software is provided
// by NIST as a service and is expressly provided "AS IS."  
// NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.
// NIST does not warrant or make any representations regarding the use of
// the software or the results thereof, including but not limited to the
// correctness, accuracy, reliability or usefulness of the software.
// 
// Permission to use this software is contingent upon your acceptance of 
// the terms of this agreement and upon your providing appropriate
// acknowledgments of NIST's ownership of the software.

#if !defined(__APPLICATION_MANAGER_INCLUDED__)
#define __APPLICATION_MANAGER_INCLUDED__

#include <Manager.h>
#include <Card_Service_Interface.h>
#include <list>


#include <APDU_Packet.h>
#include <Octet_String.h>

// An APDU is a persistent Octet_String
// that knows how to interface with APDU_Packet
class APDU : public Octet_String
{
public:
  void store_into_APDU_packet ( APDU_Packet & packet ) const
    {
      packet . set_APDU_length ( length ( ) ) ;
      Octet_String::cpy ( packet . APDU ( ) , 
                          static_cast < Memory::size_t > 
                              ( packet . max_APDU_size ( ) )  , 
                          * this ,
                          length ( ) );
    } ;

  APDU ( const APDU_Packet & packet )
    : Octet_String ( packet . APDU ( ) , 
                     static_cast < Memory::size_t > 
                         ( packet . get_APDU_length ( ) ) )
    { } ;

  APDU ( const Octet_String & data_block )
    : Octet_String ( data_block )
    { } ;
} ;


class Application_Manager: public Manager
{
public:
  typedef list < Octet_String , persistent_allocator < Octet_String > > Load_Module ;

  virtual long store_load_module ( const unsigned long flags,
                                   const Octet_String & module_AID, 
                                   const Load_Module & data_blocks ) = 0 ;

  virtual long connect_runnable_instance ( const Octet_String & module_AID ) = 0 ;

  virtual long create_runnable_instance ( const unsigned long flags ,
                                          const Octet_String & module_AID ,
                                          const Octet_String & requested_instance_AID ,
                                          const Octet_String & install_parameters ,
                                          Octet_String & created_instance_AID ) = 0 ;

  // Application Managers use this pointer to access the Card Manager API Services.
  Card_Service_Interface * card_service;

};

typedef Application_Manager::Load_Module Load_Module ;

#endif  // __APPLICATION_MANAGER_INCLUDED__