// TLVs in the FCP
//
// COPYRIGHT
// ---------
// The software was developed by the National Institute of Standards and 
// Technology (NIST), an agency of the Federal Government.  Pursuant to 
// Title 15 United States Code Section 105, works of NIST are not subject
// to copyright protection in the United States and are considered to be
// in the public domain.  As a result, a formal license is not needed to
// use the software.
// 
// The Government Smart Card Reference Implementation software is provided
// by NIST as a service and is expressly provided "AS IS."  
// NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.
// NIST does not warrant or make any representations regarding the use of
// the software or the results thereof, including but not limited to the
// correctness, accuracy, reliability or usefulness of the software.
// 
// Permission to use this software is contingent upon your acceptance of 
// the terms of this agreement and upon your providing appropriate
// acknowledgments of NIST's ownership of the software.

#include <macros.h>
#include <tags.h>

/*
** Useful FCP Macros
*/

#define DIRECTORY_FCP_ALWAYS(fid) \
                                 { FCP_TAG : \
                                    { FCP_FDB : FDB_SHAREABLE_DF , DCB_PROPRIETARY_BYTES } , \
                                    { FCP_FID : NETWORK_WORD_BYTES(fid) } , \
                                    { FCP_LIFECYCLE : LCS_OPERATIONAL_ACTIVATED } , \
                                    { ACR_EXPANDED_FORM : \
                                        { ACR_SC_DO : ACR_AM_DIR_ANY }, \
                                        { ACR_SC_DO_ALWAYS : } \
                                    }, \
                                    { FCP_FILEALLOC : NETWORK_WORD_BYTES(0) } \
                                 } 

#define DIRECTORY_FCP_PIN_ANY(fid) \
                                 { FCP_TAG : \
                                    { FCP_FDB : FDB_SHAREABLE_DF , DCB_PROPRIETARY_BYTES } , \
                                    { FCP_FID : NETWORK_WORD_BYTES(fid) } , \
                                    { FCP_LIFECYCLE : LCS_OPERATIONAL_ACTIVATED } , \
                                    { ACR_EXPANDED_FORM : { ACR_SC_DO : ACR_AM_FILE_ANY }, \
                                       { ACR_SC_DO  : { ACR_SC_KEY_REFERENCE : 0x01 }, { ACR_SC_USAGE_QUALIFIER : ACR_UQ_USE } } \
                                    }, \
                                    { FCP_FILEALLOC : NETWORK_WORD_BYTES(0) } \
                                 }

#define TRANSPARENT_FCP_ALWAYS(fid) \
                                 { FCP_TAG : { FCP_FDB : FDB_SHAREABLE_TRANSPARENT , DCB_PROPRIETARY_BYTES } , \
                                 { FCP_FID : NETWORK_WORD_BYTES(fid) } , \
                                 { FCP_LIFECYCLE : LCS_OPERATIONAL_ACTIVATED } , \
                                 { FCP_FILESIZE : 0 , 0 } , \
                                 { ACR_EXPANDED_FORM : { ACR_SC_DO : ACR_AM_FILE_ANY }, { ACR_SC_DO_ALWAYS : } }, \
                                 { FCP_FILEALLOC : NETWORK_WORD_BYTES(64) } } 

#define TRANSPARENT_LARGE_FCP_ALWAYS(fid, size) \
                                 { FCP_TAG : { FCP_FDB : FDB_SHAREABLE_TRANSPARENT , DCB_PROPRIETARY_BYTES } , \
                                 { FCP_FID : NETWORK_WORD_BYTES(fid) } , \
                                 { FCP_LIFECYCLE : LCS_OPERATIONAL_ACTIVATED } , \
                                 { FCP_FILESIZE : 0 , 0 } , \
                                 { ACR_EXPANDED_FORM : { ACR_SC_DO : ACR_AM_FILE_ANY }, { ACR_SC_DO_ALWAYS : } }, \
                                 { FCP_FILEALLOC : NETWORK_WORD_BYTES(size) } } 

#define TRANSPARENT_EXTENDED_FCP_ALWAYS(fid, kilobytes)  \
                                 { FCP_TAG : { FCP_FDB : FDB_EXTENDED_TRANSPARENT , DCB_PROPRIETARY_BYTES } , \
                                 { FCP_FID : NETWORK_WORD_BYTES(fid) } , \
                                 { FCP_LIFECYCLE : LCS_OPERATIONAL_ACTIVATED } , \
                                 { FCP_FILESIZE : 0 , 0 } , \
                                 { ACR_EXPANDED_FORM : { ACR_SC_DO : ACR_AM_FILE_ANY }, { ACR_SC_DO_ALWAYS : } }, \
                                 { FCP_FILEALLOC : NETWORK_WORD_BYTES(kilobytes) } } 

#define TRANSPARENT_LARGE_FCP_ADWRITE(fid) \
                                 { FCP_TAG : { FCP_FDB : 0x41 , 0x21 } , \
                                 { FCP_FID : NETWORK_WORD_BYTES(fid) } , \
                                 { FCP_LIFECYCLE : LCS_OPERATIONAL_ACTIVATED } , \
                                 { FCP_FILESIZE : 0 , 0 } , \
                                 { FCP_ACR_EXPANDED : { ACR_SC_DO : ACR_AM_FILE_READ }, { ACR_SC_DO_ALWAYS : }, \
                                                      { ACR_SC_DO : ACR_AM_FILE_UPDATE }, \
                                                      { ACR_SC_DO  : { ACR_SC_KEY_REFERENCE : 0x0A }, { ACR_SC_USAGE_QUALIFIER : ACR_UQ_USE } } }, \
                                 { FCP_FILEALLOC : NETWORK_WORD_BYTES(512) } } 

#define TRANSPARENT_FCP_NEVER(fid) \
                                 { FCP_TAG : { FCP_FDB : FDB_SHAREABLE_TRANSPARENT , DCB_PROPRIETARY_BYTES } , \
                                 { FCP_FID : NETWORK_WORD_BYTES(fid) } , \
                                 { FCP_LIFECYCLE : LCS_OPERATIONAL_ACTIVATED } , \
                                 { FCP_FILESIZE : 0 , 0 } , \
                                 { FCP_ACR_EXPANDED : { ACR_SC_DO : ACR_AM_FILE_ANY }, { ACR_SC_DO_NEVER : } }, \
                                 { FCP_FILEALLOC : NETWORK_WORD_BYTES(64) } } 

#define TRANSPARENT_FCP_PIN_ANY(fid,size) \
                                 { FCP_TAG : { FCP_FDB : FDB_SHAREABLE_TRANSPARENT , DCB_PROPRIETARY_BYTES } , \
                                 { FCP_FID : NETWORK_WORD_BYTES(fid) } , \
                                 { FCP_LIFECYCLE : LCS_OPERATIONAL_ACTIVATED } , \
                                 { FCP_FILESIZE : 0 , 0 } , \
                                 { FCP_ACR_EXPANDED : { ACR_SC_DO : ACR_AM_FILE_ANY }, \
                                                      { ACR_SC_DO  : { ACR_SC_KEY_REFERENCE : 0x01 }, { ACR_SC_USAGE_QUALIFIER : ACR_UQ_USE } } }, \
                                 { FCP_FILEALLOC : NETWORK_WORD_BYTES(size) } } 

#define TRANSPARENT_FCP_ADANY(fid) \
                                 { FCP_TAG : { FCP_FDB : 0x41 , 0x21 } , \
                                 { FCP_FID : NETWORK_WORD_BYTES(fid) } , \
                                 { FCP_LIFECYCLE : LCS_OPERATIONAL_ACTIVATED } , \
                                 { FCP_FILESIZE : 0 , 0 } , \
                                 { FCP_ACR_EXPANDED : { ACR_SC_DO : ACR_AM_FILE_ANY }, \
                                                      { ACR_SC_DO  : { ACR_SC_KEY_REFERENCE : 0x0A }, { ACR_SC_USAGE_QUALIFIER : ACR_UQ_USE } } }, \
                                 { FCP_FILEALLOC : NETWORK_WORD_BYTES(64) } } 

#define TRANSPARENT_FCP_PIN_READ(fid) \
                                 { FCP_TAG : { FCP_FDB : FDB_SHAREABLE_TRANSPARENT , DCB_PROPRIETARY_BYTES } , \
                                 { FCP_FID : NETWORK_WORD_BYTES(fid) } , \
                                 { FCP_LIFECYCLE : LCS_OPERATIONAL_ACTIVATED } , \
                                 { FCP_FILESIZE : 0 , 0 } , \
                                 { FCP_ACR_EXPANDED : { ACR_SC_DO : ACR_AM_FILE_READ }, \
                                                      { ACR_SC_DO  : { ACR_SC_KEY_REFERENCE : 0x01 }, { ACR_SC_USAGE_QUALIFIER : ACR_UQ_USE } } }, \
                                 { FCP_FILEALLOC : NETWORK_WORD_BYTES(64) } }

#define TRANSPARENT_FCP_PIN_WRITE(fid) \
                                 { FCP_TAG : { FCP_FDB : FDB_SHAREABLE_TRANSPARENT , DCB_PROPRIETARY_BYTES } , \
                                 { FCP_FID : NETWORK_WORD_BYTES(fid) } , \
                                 { FCP_LIFECYCLE : LCS_OPERATIONAL_ACTIVATED } , \
                                 { FCP_FILESIZE : 0 , 0 } , \
                                 { FCP_ACR_EXPANDED : { ACR_SC_DO : ACR_AM_FILE_UPDATE }, \
                                                      { ACR_SC_DO  : { ACR_SC_KEY_REFERENCE : 0x01 }, { ACR_SC_USAGE_QUALIFIER : ACR_UQ_USE } } }, \
                                 { FCP_FILEALLOC : NETWORK_WORD_BYTES(64) } }

#define TRANSPARENT_FCP_PIN_OR_ADANY(fid) \
                                 { FCP_TAG : { FCP_FDB : FDB_SHAREABLE_TRANSPARENT , DCB_PROPRIETARY_BYTES } , \
                                 { FCP_FID : NETWORK_WORD_BYTES(fid) } , \
                                 { FCP_LIFECYCLE : LCS_OPERATIONAL_ACTIVATED } , \
                                 { FCP_FILESIZE : 0 , 0 } , \
                                 { FCP_ACR_EXPANDED : { ACR_SC_DO : ACR_AM_FILE_ANY }, \
                                                      { ACR_SC_DO_OR  : \
                                                      { ACR_SC_DO  : { ACR_SC_KEY_REFERENCE : 0x01 }, { ACR_SC_USAGE_QUALIFIER : ACR_UQ_USE } }, \
                                                      { ACR_SC_DO  : { ACR_SC_KEY_REFERENCE : 0x0A }, { ACR_SC_USAGE_QUALIFIER : ACR_UQ_USE } } } }, \
                                 { FCP_FILEALLOC : NETWORK_WORD_BYTES(64) } }

#define TRANSPARENT_FCP_PIN_AND_ADANY(fid) \
                                 { FCP_TAG : { FCP_FDB : FDB_SHAREABLE_TRANSPARENT , DCB_PROPRIETARY_BYTES } , \
                                 { FCP_FID : NETWORK_WORD_BYTES(fid) } , \
                                 { FCP_LIFECYCLE : LCS_OPERATIONAL_ACTIVATED } , \
                                 { FCP_FILESIZE : 0 , 0 } , \
                                 { FCP_ACR_EXPANDED : { ACR_SC_DO : ACR_AM_FILE_ANY }, \
                                                      { ACR_SC_DO_AND  : \
                                                      { ACR_SC_DO  : { ACR_SC_KEY_REFERENCE : 0x01 }, { ACR_SC_USAGE_QUALIFIER : ACR_UQ_USE } }, \
                                                      { ACR_SC_DO  : { ACR_SC_KEY_REFERENCE : 0x0A }, { ACR_SC_USAGE_QUALIFIER : ACR_UQ_USE } } } }, \
                                 { FCP_FILEALLOC : NETWORK_WORD_BYTES(64) } }


#define LINEAR_RECORD_FCP_ALWAYS(fid, recsize, numrecs ) \
                                 { FCP_TAG : \
                                   { FCP_FDB : FDB_LINEAR_FIXED, \
                                               DCB_PROPRIETARY_BYTES, \
                                               NETWORK_WORD_BYTES(recsize), \
                                               NETWORK_WORD_BYTES(numrecs) }, \
                                   { FCP_FID : NETWORK_WORD_BYTES(fid) } , \
                                   { FCP_LIFECYCLE : LCS_OPERATIONAL_ACTIVATED } , \
                                   { FCP_FILESIZE : NETWORK_WORD_BYTES(0) } , \
                                   { FCP_ACR_EXPANDED : { ACR_SC_DO : ACR_AM_FILE_ANY }, { ACR_SC_DO_ALWAYS : } }, \
                                   { FCP_FILEALLOC : NETWORK_WORD_BYTES(64) } } 

#define LINEAR_TLV_RECORD_FCP_ALWAYS(fid, recsize, numrecs ) \
                                 { FCP_TAG : \
                                   { FCP_FDB : FDB_LINEAR_FIXED_TLV, \
                                               DCB_PROPRIETARY_BYTES, \
                                               NETWORK_WORD_BYTES(recsize), \
                                               NETWORK_WORD_BYTES(numrecs) }, \
                                   { FCP_FID : NETWORK_WORD_BYTES(fid) } , \
                                   { FCP_LIFECYCLE : LCS_OPERATIONAL_ACTIVATED } , \
                                   { FCP_FILESIZE : NETWORK_WORD_BYTES(0) } , \
                                   { FCP_ACR_EXPANDED : { ACR_SC_DO : ACR_AM_FILE_ANY }, { ACR_SC_DO_ALWAYS : } }, \
                                   { FCP_FILEALLOC : NETWORK_WORD_BYTES(64) } } 

#define TRANSPARENT_FCP_ARR(fid, arr, recno ) \
                                 { FCP_TAG : { FCP_FDB : FDB_SHAREABLE_TRANSPARENT , DCB_PROPRIETARY_BYTES } , \
                                 { FCP_FID : NETWORK_WORD_BYTES(fid) } , \
                                 { FCP_LIFECYCLE : LCS_OPERATIONAL_ACTIVATED } , \
                                 { FCP_FILESIZE : NETWORK_WORD_BYTES(0) } , \
                                 { FCP_ACR_REFERENCE : NETWORK_WORD_BYTES(arr), recno }, \
                                 { FCP_FILEALLOC : NETWORK_WORD_BYTES(64) } } 

#define TRANSPARENT_FCP_ARR_SEID(fid, arr, seid, recno ) \
                                 { FCP_TAG : { FCP_FDB : FDB_SHAREABLE_TRANSPARENT , DCB_PROPRIETARY_BYTES } , \
                                 { FCP_FID : NETWORK_WORD_BYTES(fid) } , \
                                 { FCP_LIFECYCLE : LCS_OPERATIONAL_ACTIVATED } , \
                                 { FCP_FILESIZE : NETWORK_WORD_BYTES(0) } , \
                                 { FCP_ACR_REFERENCE : NETWORK_WORD_BYTES(arr), seid, recno }, \
                                 { FCP_FILEALLOC : NETWORK_WORD_BYTES(64) } }

#define TRANSPARENT_FCP_EF_ARR(fid, recno ) TRANSPARENT_FCP_ARR(fid, EF_ARR, recno )

#define LINEAR_RECORD_FCP_EF_ARR(fid, recsize, numrecs, recno ) \
                                 { FCP_TAG : \
                                   { FCP_FDB : FDB_LINEAR_FIXED, \
                                               DCB_PROPRIETARY_BYTES, \
                                               NETWORK_WORD_BYTES(recsize), \
                                               NETWORK_WORD_BYTES(numrecs) }, \
                                   { FCP_FID : NETWORK_WORD_BYTES(fid) } , \
                                   { FCP_LIFECYCLE : LCS_OPERATIONAL_ACTIVATED } , \
                                   { FCP_FILESIZE : NETWORK_WORD_BYTES(0) } , \
                                   { FCP_ACR_REFERENCE : NETWORK_WORD_BYTES(EF_ARR), recno }, \
                                   { FCP_FILEALLOC : NETWORK_WORD_BYTES(64) } }

#define ARR_FCP_ARR(fid, recsize, numrecs) \
                        {         { FCP_TAG : \
                                   { FCP_FDB : FDB_LINEAR_FIXED_TLV, \
                                               DCB_PROPRIETARY_BYTES, \
                                               NETWORK_WORD_BYTES(recsize), \
                                               NETWORK_WORD_BYTES(numrecs) }, \
                                   { FCP_FID : NETWORK_WORD_BYTES(fid) } , \
                                   { FCP_LIFECYCLE : LCS_OPERATIONAL_ACTIVATED } , \
                                   { FCP_FILESIZE : NETWORK_WORD_BYTES(0) } , \
                                   { FCP_ACR_REFERENCE : NETWORK_WORD_BYTES(fid), 1 }, \
                                   { FCP_FILEALLOC : NETWORK_WORD_BYTES(64) } } , \
                                  { FS_TRANSPARENT_BER_TLV_DATA : \
                                    {  ACR_EXPANDED_FORM : { ACR_SC_DO : ACR_AM_FILE_ANY }, \
                                                           { ACR_SC_DO_ALWAYS : } } } }

#define SPECIAL_FCP_ADANY(fid) \
                                 { FCP_TAG : { FCP_FDB : FDB_SPECIAL , DCB_PROPRIETARY_BYTES } , \
                                 { FCP_FID : NETWORK_WORD_BYTES(fid) } , \
                                 { FCP_LIFECYCLE : LCS_OPERATIONAL_ACTIVATED } , \
                                 { FCP_FILESIZE : 0 , 0 } , \
                                 { FCP_ACR_EXPANDED : { ACR_SC_DO : ACR_AM_FILE_ANY }, \
                                                      { ACR_SC_DO  : { ACR_SC_KEY_REFERENCE : 0x0A },  \
                                                                     { ACR_SC_USAGE_QUALIFIER : ACR_UQ_USE } } }, \
                                 { FCP_FILEALLOC : NETWORK_WORD_BYTES(64) } }

#define RSA_KEY_PAIR_FCP_ADWRITE(fid) \
                                 { FCP_TAG : { FCP_FDB : 0x41 , 0x21 } , \
                                 { FCP_FID : NETWORK_WORD_BYTES(fid) } , \
                                 { FCP_LIFECYCLE : LCS_OPERATIONAL_ACTIVATED } , \
                                 { FCP_FILESIZE : 0 , 0 } , \
                                 { FCP_ACR_EXPANDED : { ACR_SC_DO : ACR_AM_FILE_READ }, { ACR_SC_DO_ALWAYS : }, \
                                                      { ACR_SC_DO : ACR_AM_FILE_UPDATE }, \
                                                      { ACR_SC_DO  : { ACR_SC_KEY_REFERENCE : 0x0A }, \
                                                                     { ACR_SC_USAGE_QUALIFIER : ACR_UQ_USE } } }, \
                                 { FCP_FILEALLOC : NETWORK_WORD_BYTES(612) } } 

