// File Manager
//
// COPYRIGHT
// ---------
// The software was developed by the National Institute of Standards and 
// Technology (NIST), an agency of the Federal Government.  Pursuant to 
// Title 15 United States Code Section 105, works of NIST are not subject
// to copyright protection in the United States and are considered to be
// in the public domain.  As a result, a formal license is not needed to
// use the software.
// 
// The Government Smart Card Reference Implementation software is provided
// by NIST as a service and is expressly provided "AS IS."  
// NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.
// NIST does not warrant or make any representations regarding the use of
// the software or the results thereof, including but not limited to the
// correctness, accuracy, reliability or usefulness of the software.
// 
// Permission to use this software is contingent upon your acceptance of 
// the terms of this agreement and upon your providing appropriate
// acknowledgments of NIST's ownership of the software.

#if !defined(__FILE_MANAGER_H_INCLUDED__)
#define __FILE_MANAGER_H_INCLUDED__

#include <map>
#include <set>

#include <APDU_Server.h>
#include <Manager.h>
#include <Security_System.h>
#include <File_System.h>

#include < Application.h >

#if defined(_MSC_VER )
  // It's OK that the compiler can't automatically generate default constructors or
  // assignment operators for classes with const data members
  // We're shooting for immutability here
  #pragma warning ( disable : 4702 )  // unreachable code in <list>
  #if defined(FILE_MANAGER_EXPORTS)
    #define FILE_MANAGER_DLL _declspec(dllexport)
  #else //  defined ( FILE_MANAGER_EXPORTS )
    #define FILE_MANAGER_DLL _declspec(dllimport)
  #endif  // defined ( FILE_MANAGER_EXPORTS )
#else
   #define FILE_MANAGER_DLL
#endif



class FILE_MANAGER_DLL File_Manager: public Application_Manager
{
private:
  Octet_String security_status() ;

public:
  friend class Application ;

  virtual long initialize      ( int v ) ;
  virtual APDU_Handler exchange_APDUs    ;
  virtual long finalize        ( void )  ;

  virtual long store_load_module ( const unsigned long flags,
                                   const Octet_String & module_AID, 
                                   const Load_Module & data_blocks ) ;

  virtual long create_runnable_instance ( const unsigned long flags ,
                                          const Octet_String & module_AID ,
                                          const Octet_String & requested_instance_AID ,
                                          const Octet_String & install_parameters ,
                                          Octet_String & created_instance_AID ) ;

  virtual long connect_runnable_instance ( const Octet_String & module_AID ) ;

  virtual long power_up         ( void ) ;
  virtual long power_down       ( void ) ;

  virtual bool store_persistent_state ( FILE * the_persistent_storage_output_file , 
                                        unsigned short & checksum ) ;
  virtual bool load_persistent_state ( FILE * the_persistent_storage_input_file , 
                                       unsigned short & checksum ) ;

  virtual void 
  get_ATR ( unsigned char * ATR , 
            unsigned int & ATR_length ) ;


  void record_APDU_handler ( CLAINS clains , 
                             Application * appplication_pointer )
    {
      the_dispatch_map . insert 
        ( Dispatch_Map::value_type ( clains , appplication_pointer ) ) ;
    } ;

  ~File_Manager ( )
    {
      // Avoid memory leak by cleaning up dispatch table references
      typedef set < Application * > Application_Pointer_Set ; 
      Application_Pointer_Set the_application_pointers ;
      for ( Dispatch_Map::iterator p = the_dispatch_map . begin ( ) ;
            p != the_dispatch_map . end ( ) ;
            p ++ )
        the_application_pointers . insert ( p -> second ) ;

      for ( Application_Pointer_Set::iterator p = the_application_pointers . begin ( ) ;
            p != the_application_pointers . end ( ) ;
            p ++ )
        free ( * p ) ;
    } ;

  // Debugging
  void dump_file_system ( ) { the_file_system_pointer -> dump ( ) ; } ;

  void create_initial_file_system ( ) ;

  APDU_Handler exchange_plaintext_APDUs    ;

  typedef map < CLAINS , Application * > Dispatch_Map ;

  typedef map < Octet_String , Octet_String > AID_to_DLL_Name_Map;

  // Persistent state below
  File_System_Pointer the_file_system_pointer ;
  AID_to_DLL_Name_Map the_AID_to_DLL_name_map ;
  // Persistent state above

  // These can be derived from persistent state above
  Security_System_Pointer the_security_system_pointer ;
  Dispatch_Map the_dispatch_map ;


  bool is_known_AID ( const Octet_String & AID )
    {
      AID_to_DLL_Name_Map::const_iterator p =
        the_AID_to_DLL_name_map . find ( AID ) ;

      if ( p != the_AID_to_DLL_name_map . end ( ) )
        return true ;
      return false ;
    } ;

  Octet_String find_DLL_for_AID ( const Octet_String & AID )
    {
      AID_to_DLL_Name_Map::const_iterator p =
        the_AID_to_DLL_name_map . find ( AID ) ;

      if ( p != the_AID_to_DLL_name_map . end ( ) )
        return p -> second ;
      
      return Octet_String ( 0 ) ;
    } ;

  Octet_String_Pointer the_challenge_pointer ;


} ;

#endif  // __FILE_MANAGER_H_INCLUDED__