// File System Application
//
// COPYRIGHT
// ---------
// The software was developed by the National Institute of Standards and 
// Technology (NIST), an agency of the Federal Government.  Pursuant to 
// Title 15 United States Code Section 105, works of NIST are not subject
// to copyright protection in the United States and are considered to be
// in the public domain.  As a result, a formal license is not needed to
// use the software.
// 
// The Government Smart Card Reference Implementation software is provided
// by NIST as a service and is expressly provided "AS IS."  
// NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.
// NIST does not warrant or make any representations regarding the use of
// the software or the results thereof, including but not limited to the
// correctness, accuracy, reliability or usefulness of the software.
// 
// Permission to use this software is contingent upon your acceptance of 
// the terms of this agreement and upon your providing appropriate
// acknowledgments of NIST's ownership of the software.

#if !defined(__FILE_SYSTEM_APPLICATION_H_INCLUDED__)
#define __FILE_SYSTEM_APPLICATION_H_INCLUDED__

#if defined(_MSC_VER )
  // It's OK that the compiler can't automatically generate default constructors or
  // assignment operators for class FILE_SYSTEM_APPLICATION_DLLes with const data members
  // We're shooting for immutability here
  #if defined(FILE_SYSTEM_APPLICATION_EXPORTS)
    #define FILE_SYSTEM_APPLICATION_DLL _declspec(dllexport)
  #else //  defined ( FILE_SYSTEM_APPLICATION_EXPORTS )
    #define FILE_SYSTEM_APPLICATION_DLL _declspec(dllimport)
  #endif  // defined ( FILE_SYSTEM_APPLICATION_EXPORTS )
#else
   #define FILE_SYSTEM_APPLICATION_DLL
#endif

#include <File_System.h>
#include <Application.h>

class FILE_SYSTEM_APPLICATION_DLL File_System_Application : public Application
{

public:
  File_System_Application( File_Manager * file_manager_pointer ,
                           const File_System_Pointer & file_system_pointer ,
                           const Security_System_Pointer & security_system_pointer ,
                           Octet_String_Pointer & challenge_pointer ) ;

  virtual long exchange_APDUs   ( const CAPDU_Packet & command , 
                                        RAPDU_Packet & response ) ;

  virtual const Octet_String & AID ( ) { return the_AID ; }

private:
 
  typedef struct { CLAINS clains ; APDU_Handler File_System_Application::* APDU_handler ; } Dispatch_Table_Entry ;
  static Dispatch_Table_Entry dispatch_table [ ] ;

  APDU_Handler read_binary                 ;
  APDU_Handler select                      ;
  APDU_Handler update_binary               ;
  APDU_Handler external_authenticate       ;
  APDU_Handler get_challenge               ;
  APDU_Handler internal_authenticate       ;
  APDU_Handler verify_pin                  ;
  APDU_Handler manage_security_environment ;
  APDU_Handler perform_security_operation  ;

  Octet_String the_AID ;
};


extern "C" FILE_SYSTEM_APPLICATION_DLL Application_Creator create_application ;

#endif  // __FILE_SYSTEM_APPLICATION_H_INCLUDED__
