// Java Card Manager
//
// COPYRIGHT
// ---------
// The software was developed by the National Institute of Standards and 
// Technology (NIST), an agency of the Federal Government.  Pursuant to 
// Title 15 United States Code Section 105, works of NIST are not subject
// to copyright protection in the United States and are considered to be
// in the public domain.  As a result, a formal license is not needed to
// use the software.
// 
// The Government Smart Card Reference Implementation software is provided
// by NIST as a service and is expressly provided "AS IS."  
// NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.
// NIST does not warrant or make any representations regarding the use of
// the software or the results thereof, including but not limited to the
// correctness, accuracy, reliability or usefulness of the software.
// 
// Permission to use this software is contingent upon your acceptance of 
// the terms of this agreement and upon your providing appropriate
// acknowledgments of NIST's ownership of the software.

#if !defined(__JAVA_CARD_MANAGER_H_INCLUDED__)
#define __JAVA_CARD_MANAGER_H_INCLUDED__

#include <Manager.h>
#include <TLV.h>
#include <Card_Service_Interface.h>
#include <TLP_APDU_Server.h>
#include <Security_System.h>

class Java_Card_Manager: public Application_Manager
  {
  private:

  public:
    Java_Card_Manager ( ) ;

    virtual long initialize      ( int v ) ;
    virtual APDU_Handler exchange_APDUs    ;
    virtual long finalize        ( void )  ;

    virtual long store_load_module ( const unsigned long flags,
                                    const Octet_String & module_AID,
                                    const Load_Module & data_blocks ) ;

    virtual long create_runnable_instance ( const unsigned long flags ,
                                            const Octet_String & module_AID ,
                                            const Octet_String & requested_instance_AID ,
                                            const Octet_String & install_parameters ,
                                            Octet_String & created_instance_AID ) ;

    virtual long connect_runnable_instance ( const Octet_String & module_AID ) ;


    virtual long power_up         ( void ) ;
    virtual long power_down       ( void ) ;
    virtual void 
    get_ATR ( unsigned char * ATR , 
              unsigned int & ATR_length )
      { the_tlp_apdu_server . get_ATR ( ATR , ATR_length ) ; } ;

  private:
    TLP_APDU_Server the_tlp_apdu_server;
    static const Octet select_OCI_command [ ] ;
    static const int CLAINS_CREATE_APPLET_INSTANCE = 0x80B8 ;
  };

#endif  // __JAVA_CARD_MANAGER_H_INCLUDED__
