// Tags, RIDs, etc.
//
// COPYRIGHT
// ---------
// The software was developed by the National Institute of Standards and 
// Technology (NIST), an agency of the Federal Government.  Pursuant to 
// Title 15 United States Code Section 105, works of NIST are not subject
// to copyright protection in the United States and are considered to be
// in the public domain.  As a result, a formal license is not needed to
// use the software.
// 
// The Government Smart Card Reference Implementation software is provided
// by NIST as a service and is expressly provided "AS IS."  
// NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.
// NIST does not warrant or make any representations regarding the use of
// the software or the results thereof, including but not limited to the
// correctness, accuracy, reliability or usefulness of the software.
// 
// Permission to use this software is contingent upon your acceptance of 
// the terms of this agreement and upon your providing appropriate
// acknowledgments of NIST's ownership of the software.

#if !defined(__TAGS_H_INCLUDED__)
#define __TAGS_H_INCLUDED__


//
// Application Directory, EF-DIR (0x2F00), Tags
//
#define TEMPLATE_EFDIR                          0x61

#define EFDIR_AID                               0x4F
#define EFDIR_LABEL                             0x50
#define EFDIR_PATH                              0x51
#define EFDIR_COMMAND                           0x52

//
// FCP Tags
//
#define FCP_TAG                                 0x62
#define FMD_TAG                                 0x64
#define FCI_TAG                                 0x6F

#define FCP_FILESIZE                            0x80
#define FCP_FILEALLOC                           0x81
#define FCP_FDB                                 0x82
#define FCP_FID                                 0x83
#define FCP_DFNAME                              0x84
#define FCP_PROPRIETARY_SIMPLE                  0x85
#define FCP_ACR_PROPRIETARY                     0x86
#define FCP_FCIEXTENSION                        0x87
#define FCP_SFI                                 0x88
#define FCP_LIFECYCLE                           0x8A
#define FCP_ACR_REFERENCE                       0x8B
#define FCP_ACR_COMPACT                         0x8C
#define FCP_SE_TEMPLATES                        0x8D
#define FCP_DATA_OBJECT_TEMPLATE                0xA0
#define FCP_SECURITY_TEMPLATE_IF                0xA1
#define FCP_SFI_TO_PATH_TEMPLATE                0xA2
#define FCP_PROPRIETARY_CONSTRUCTED             0xA5
#define FCP_ACR_EXPANDED                        0xAB
#define FCP_PINSTATUS                           0xC6
#define FCP_TAG_LIST                            0x5C

//
// Tags for Standardized Data Elements in 0xA5 Template in FCP 
//
#define FCP_PROPRIETARY_CONSTRUCTED_UICC        0x80
#define FCP_PROPRIETARY_CONSTRUCTED_POWER       0x81
#define FCP_PROPRIETARY_CONSTRUCTED_CLOCK       0x82
#define FCP_PROPRIETARY_CONSTRUCTED_MEMORY      0x83

// File Descriptor Byte //
#define FDB_DF                                  0x38
#define FDB_EF                                  0x00
#define FDB_INTERNAL_EF                         0x08
#define FDB_TRANSPARENT                         0x01
#define FDB_LINEAR_FIXED                        0x02
#define FDB_LINEAR_FIXED_TLV                    0x03
#define FDB_LINEAR_VARIABLE                     0x04
#define FDB_LINEAR_VARIABLE_TLV                 0x05
#define FDB_CYCLIC                              0x06
#define FDB_CYCLIC_TLV                          0x07
#define FDB_SIMPLE_TLV_STRUCTURE                0x39
#define FDB_BER_TLV_STRUCTURE                   0x3A

#define FDB_CATEGORY                            0x38
#define FDB_TYPE                                0x3F
#define FDB_SHAREABLE                           0x40

#define FDB_EXECUTABLE                          0x10


// Combinations of the above
#define FDB_SHAREABLE_DF                        ( FDB_SHAREABLE | FDB_DF              )
#define FDB_SHAREABLE_TRANSPARENT               ( FDB_SHAREABLE | FDB_TRANSPARENT     )
#define FDB_SHAREABLE_LINEAR_FIXED              ( FDB_SHAREABLE | FDB_LINEAR_FIXED    )
#define FDB_SHAREABLE_LINEAR_VARIABLE           ( FDB_SHAREABLE | FDB_LINEAR_VARIABLE )
#define FDB_SHAREABLE_CYCLIC                    ( FDB_SHAREABLE | FDB_CYCLIC )
#define FDB_SHAREABLE_SIMPLE_TLV                ( FDB_SHAREABLE | FDB_SIMPLE_TLV_STRUCTURE )
#define FDB_SHAREABLE_BER_TLV                   ( FDB_SHAREABLE | FDB_BER_TLV_STRUCTURE )

#define FDB_INTERNAL_TRANSPARENT                ( FDB_INTERNAL_EF | FDB_TRANSPARENT     )
#define FDB_INTERNAL_LINEAR_FIXED               ( FDB_INTERNAL_EF | FDB_LINEAR_FIXED    )
#define FDB_INTERNAL_LINEAR_VARIABLE            ( FDB_INTERNAL_EF | FDB_LINEAR_VARIABLE )
#define FDB_INTERNAL_CYCLIC                     ( FDB_INTERNAL_EF | FDB_CYCLIC )

// Data Coding Byte //
#define DCB_PROPRIETARY_WRITE_BEHAVIOR          0x20
#define DCB_DATA_UNIT_QUARTETS                  0x01
#define DCB_PROPRIETARY_BYTES                   ( DCB_PROPRIETARY_WRITE_BEHAVIOR | DCB_DATA_UNIT_QUARTETS )

// Life Cycle Status //
#define LCS_CREATION                            0x01
#define LCS_INITIALIZATION                      0x03
#define LCS_OPERATIONAL_ACTIVATED               0x05
#define LCS_OPERATIONAL_DEACTIVATED             0x04
#define LCS_TERMINATED                          0x0C

// Security Environments //
#define SECURITY_ENVIRONMENT_TEMPLATE_DO        0x7B      // 7816-8
#define SEID_BYTE                               0x80      // 7816-4, table 37
#define LCS_BYTE                                0x8A      // 7816-4, table 37

// PIN Status //
#define PIN_STATUS_FLAGS                        0x90
#define PIN_USAGE_QUALIFIER                     0x95
#define PIN_KEY_REFERENCE                       0x83

//
// Symmetric_Key Generation Data Objects
//
#define KEYGEN_KEY_SIZE                         0x80
#define KEYGEN_KEY_FILE                         0x81

//
// Data Objects in the dynamic authentication template (GENERAL AUTHENTICATE)
//
#define DYN_TEMPLATE                            0x7C
#define DYN_WITNESS                             0x80
#define DYN_CHALLENGE                           0x81
#define DYN_RESPONSE                            0x82
#define DYN_COMMITTED_CHALLENGE                 0x83
#define DYN_AUTHENTICATION_CODE                 0x84
#define DYN_EXPONENTIAL                         0x85
#define DYN_COMPUTED_CHALLENGE                  0x86
#define DYN_IDENTIFICATION_DATA_TEMPLATE        0xA0

//
// Data Objects in the Data Field of an "Odd INS" Command
//
#define DATA_FIELD_TAG_LIST                     0x5C
#define DATA_FIELD_DATA                         0x53
#define DATA_FIELD_PRIMITIVE_BER_TLV            0x53
#define DATA_FIELD_CONSTRUCTED_BER_TLV          0x73
#define DATA_FIELD_OFFSET                       0x54

//
// Data Objects in the Data Field of the GENERATE ASYMMETRIC KEY PAIR Command
//
#define GAK_TEMPLATE                            0xAC
#define GAK_MECHANISM                           0x80
#define GAK_PARAMETERS                          0x81

// Secure Messaging Data Objects
//
#define SM_PLAIN_VALUE_NOT_BER_TLV              0x80
#define SM_CRYPTOGRAM_WITH_SM_OBJECTS           0x82
#define SM_CRYPTOGRAM_WITHOUT_SM_OBJECTS        0x84
#define SM_CRYPTOGRAM_WITH_PADDING              0x86
#define SM_CRYPTOGRAPHIC_CHECKSUM               0x8E
#define SM_HASH_CODE                            0x90
#define SM_CERTIFICATE                          0x92
#define SM_LE_IN_UNSECURED_APDU                 0x96
#define SM_STATUS_INFORMATION                   0x99
#define SM_INPUT_FOR_DIGITAL_SIG_NOT_BER        0x9A
#define SM_PUBLIC_KEY                           0x9C
#define SM_DIGITAL_SIGNATURE                    0x9E
#define SM_INPUT_FOR_HASH                       0xA0
#define SM_INPUT_FOR_CHECKSUM                   0xA2
#define SM_INPUT_FOR_DIGITAL_SIGNATURE          0xA8
#define SM_INPUT_FOR_DIGITAL_SIG_CONCAT         0xAC
#define SM_INPUT_FOR_CERTIFICATE                0xAE
#define SM_PLAIN_VALUE_BER_TLV_WITH_SM_OBJ      0xB0
#define SM_PLAIN_VALUE_BER_TLV_NO_SM_OBJ        0xB2
#define SM_RESPONSE_DESCRIPTOR_TEMPLATE         0xBA
#define SM_INPUT_FOR_DIGITAL_SIG_BER_SIGN       0xBC
#define SM_INPUT_FOR_VERT_BER_SIGN              0xBE

#define SM_PLAIN_VALUE_NOT_BER_TLV_CHK          0x81
#define SM_CRYPTOGRAM_WITH_SM_OBJECTS_CHK       0x83
#define SM_CRYPTOGRAM_WITHOUT_SM_OBJECTS_CHK    0x85
#define SM_CRYPTOGRAM_WITH_PADDING_CHK          0x87
#define SM_COMMAND_HEADER_CHK                   0x89
#define SM_LE_IN_UNSECURED_APDU_CHK             0x97
#define SM_SW_CHK                               0x99
#define SM_PLAIN_VALUE_BER_TLV_WITH_SM_OBJ_CHK  0xB1
#define SM_PLAIN_VALUE_BER_TLV_NO_SM_OBJ_CHK    0xB3
#define SM_RESPONSE_DESCRIPTOR_TEMPLATE_CHK     0xBB

//
// CCC Tags
//
#define CCC_CARD_IDENTIFIER         0xF0
#define CCC_CONTAINER_VERSION       0xF1
#define CCC_GRAMMAR_VERSION         0xF2
#define CCC_CARD_URL                0xF3
#define CCC_PKCS_15                 0xF4
#define CCC_DATA_MODEL              0xF5
#define CCC_REDIRECTION             0xFA
#define CCC_CAPABILITY_TUPLES       0xFB
#define CCC_STATUS_WORD_TUPLES      0xFC
#define CCC_NEXT_CONTAINER          0xFD
#define CCC_ERROR_DETECTION         0xFE

//
// CCC Values
//

//
// Application Property Template
//
#define APT_TEMPLATE                0x61
#define APT_AID                     0x4F
#define APT_TAG_AUTHORITY           0x79
#define APT_APPLICATION_LABEL       0x50
#define APT_URL                     0x5F,0x50
#define APT_TAG_AID                 0x4F

// 
// Card Identifier
//
#define GSA_RID                     0xA0 , 0x00 , 0x00 , 0x01 , 0x16
#define NIST_RID                    0xA0 , 0x00 , 0x00 , 0x03 , 0x08

#define MFID_NIST                   0x01

#define CARD_TYPE_FILE_SYSTEM_CARD              0x01
#define CARD_TYPE_JAVA_CARD                     0x02
#define CARD_TYPE_JAVA_CARD_FS                  0x03
#define CARD_TYPE_MULTOS                        0x04
#define CARD_TYPE_DOT_NET                       0x05
#define CARD_TYPE_GENERIC_MULTI_APPLICATION     0x0F

#define CARD_ID_PIV_REF_IMP   0x50, 0x49, 0x56, 0x52, 0x65, 0x66, 0x49, 0x6D, 0x70  // PIVRefImp

// PKCS # 15 Version
#define NON_PKCS_15                    0

//
// Application Card URL
//
#define APP_TYPE_GENERIC            0x01
#define APP_TYPE_SKI                0x02
#define APP_TYPE_PKI                0x04

#define APP_TYPE_GENERIC_EXTENSION  0x08
#define APP_TYPE_SKI_EXTENSION      0x10
#define APP_TYPE_PKI_EXTENSION      0x20

//
// Registered Data Models
//
#define DATA_MODEL_GSC              0x01
#define DATA_MODEL_CAC              0x02
#define DATA_MODEL_PIV              0x10

//
// Access Controls
//
#define ACR_ALWAYS                           0x00
#define ACR_NEVER                            0x01
#define ACR_EXTERNAL_AUTHENTICATION          0x02
#define ACR_EXTERNAL_AUTHENTICATION_OR_PIN   0x03
#define ACR_SECURE_CHANNEL_GP                0x04
#define ACR_PIN_ALWAYS                       0x05
#define ACR_PIN_PROTECTED                    0x06
#define ACR_EXTERNAL_AUTHENTICATION_THEN_PIN 0x07
#define ACR_UPDATE_ONCE                      0x08
#define ACR_PIN_THEN_AUTHENTICATION          0x09
#define ACR_SECURE_CHANNEL_ISO               0x0B

//
// Capability Tuple Codes
//
#define CODE_CONSTANT                        0x00
#define CODE_DESCRIPTOR                      0x80

#define PAR_DATA                             0x00
#define PAR_CLA                              0x10
#define PAR_INS                              0x20
#define PAR_P1                               0x30
#define PAR_P2                               0x40
#define PAR_P3                               0x50
#define PAR_PREFIX                           0x60
#define PAR_SUFFIX                           0x70

#define VALUE_CONSTANT                       0x00
#define VALUE_DESCRIPTOR                     0x80

#define FUN_SELECT_DF                        0x01
#define FUN_READ_BINARY                      0x02
#define FUN_UPDATE_BINARY                    0x03
#define FUN_SECURE_UPDATE_BINARY             0x04
#define FUN_MANAGE_SECURITY_ENVIRONMENT      0x05
#define FUN_GET_CHALLENGE                    0x06
#define FUN_GET_RESPONSE                     0x07
#define FUN_VERIFY_PIN                       0x08
#define FUN_INTERNAL_AUTHENTICATE            0x09
#define FUN_EXTERNAL_AUTHENTICATE            0x0A
#define FUN_PERFORM_SECURITY_OPERATION       0x0B
#define FUN_SELECT_EF                        0x0D
#define FUN_SELECT_MF                        0x0E
#define FUN_SECURE_READ_BINARY               0x0F

#endif // !defined(__TAGS_H_INCLUDED__)
