// Messages
//
// COPYRIGHT
// ---------
// The software was developed by the National Institute of Standards and 
// Technology (NIST), an agency of the Federal Government.  Pursuant to 
// Title 15 United States Code Section 105, works of NIST are not subject
// to copyright protection in the United States and are considered to be
// in the public domain.  As a result, a formal license is not needed to
// use the software.
// 
// The Government Smart Card Reference Implementation software is provided
// by NIST as a service and is expressly provided "AS IS."  
// NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.
// NIST does not warrant or make any representations regarding the use of
// the software or the results thereof, including but not limited to the
// correctness, accuracy, reliability or usefulness of the software.
// 
// Permission to use this software is contingent upon your acceptance of 
// the terms of this agreement and upon your providing appropriate
// acknowledgments of NIST's ownership of the software.

#if !defined(__MESSAGES_H_INCLUDED__)
#define __MESSAGES_H_INCLUDED__
//--------------------------------------------------------------------------
// Debugging printout
//
// Use the debugger output console if availabe.
//
//--------------------------------------------------------------------------

//--------------------------------------------------------------------------
//
// Some notes on output strategy:
//
// Four kinds of output:
//
//    standard output
//       only applies to _CONSOLE programs that may read standard input
//
//    logging output
//       applies to all programs
//       typically throttled by "verbosity"
//       _CONSOLE programs:
//          should go to a .log or .out file, or stdout
//       GUI programs
//          should go to a .log or .out file, 
//             or debugging output window if running under a debugger
//          otherwise just silently dropped
//
//    error output
//       applies to all programs
//       _CONSOLE programs
//          should go to a .err file, or stderr
//       GUI programs
//          should go a pop-up MessageBox
//       program will often exit immediately thereafter
//
//
//    debugging output
//       applies to all programs
//       typically gated by _DEBUG and throttled by #define constants
//       _CONSOLE programs:
//          should go to stdout
//       GUI programs
//          should go to a debugging output window if running under a debugger
//          or to a MessageBox under control of a run-time flag defaulted
//             to false
//
//
// Interfaces:
//    standard output
//       stdout
//    logging output
//       log_message(Format, ...)
//    error output
//       error_message(Format, ...)
//    debugging output
//       debug_message(Format, ...)
//    
//
// Implementations:
//
//    standard output
//       FILE * stdout
//
//    logging output
//       _CONSOLE programs
//          FILE * logfile -- defined here
//       GUI programs
//          OutputDebugString
//
//    error output
//       _CONSOLE programs
//          FILE * stderr
//       GUI programs
//          MessageBox
//
//    debugging output
//       _CONSOLE programs
//          FILE * stdout
//       GUI programs
//          OutputDebugString or MessageBox, with UseMessageBox
//    
//--------------------------------------------------------------------------



#if defined(__cplusplus)
   extern "C" {
#endif

#include <stdio.h>
#include <stdarg.h>

extern FILE * logfile ;

#if !defined(_CONSOLE)
   extern int UseMessageBox ;
#endif   // defined(_CONSOLE)

extern void log_message(const char * Format, ...) ;

extern void error_message(const char * Format, ...) ;

extern void debug_message(const char * Format, ...) ;

#if defined(__cplusplus)
     }  //  extern "C"
#endif
#endif // !defined(__MESSAGES_H_INCLUDED__)
