// TLP_APDU_Client -- TLP224 Listener
//
// COPYRIGHT
// ---------
// The software was developed by the National Institute of Standards and 
// Technology (NIST), an agency of the Federal Government.  Pursuant to 
// Title 15 United States Code Section 105, works of NIST are not subject
// to copyright protection in the United States and are considered to be
// in the public domain.  As a result, a formal license is not needed to
// use the software.
// 
// The Government Smart Card Reference Implementation software is provided
// by NIST as a service and is expressly provided "AS IS."  
// NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR STATUTORY, 
// INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.
// NIST does not warrant or make any representations regarding the use of
// the software or the results thereof, including but not limited to the
// correctness, accuracy, reliability or usefulness of the software.
// 
// Permission to use this software is contingent upon your acceptance of 
// the terms of this agreement and upon your providing appropriate
// acknowledgments of NIST's ownership of the software.

#if !defined(__TLP_APDU_CLIENT_H_INCLUDED__)
#define __TLP_APDU_CLIENT_H_INCLUDED__

// This listener is a client of an APDU server.
#include <APDU_Client.h>
#include <TLP224.h>

class TLP_APDU_Client : public APDU_Client
  {
  public:
    static const int DEFAULT_APDU_PORT = 9025 ;
    static const int MAX_HOSTNAME_BUFFER_LENGTH = 64;  
    static const char default_hostname [ ];

    TLP_APDU_Client ( APDU_Server & server ) 
      : APDU_Client ( server  ) ,
        the_port ( DEFAULT_APDU_PORT )  ,
        the_host ( default_hostname ) ,
        the_listening_socket ( INVALID_SOCKET ) ,
        the_connection_socket ( INVALID_SOCKET ) ,
        the_current_logical_card_identifier 
         ( static_cast < unsigned short > ( - 1 ) ) ,
        the_verbosity ( 0 ) ,
        use_GET_RESPONSE ( true )
      { } ;

    int get_port ( ) 
      { 
        return the_port ; 
      } ;

    int & set_port ( int p ) 
      { 
        the_port = p ; 
        return the_port  ; 
      } ;

    const char * get_host ( ) 
      { 
        return the_host ; 
      } ;

    const char * set_host ( const char * host ) 
      { 
        return the_host = host ; 
      } ;

    int get_current_card_number ( ) 
      {
        return the_current_logical_card_identifier ;
      } ;

  virtual long initialize ( int verbosity ) ;

  virtual long run ( void ) ;

  virtual long finalize ( void ) ;

  bool set_use_GET_RESPONSE ( bool b ) { return use_GET_RESPONSE = b ; } ; 

  bool get_use_GET_RESPONSE ( ) { return use_GET_RESPONSE ; } ; 

  private:
    int the_port ;
    const char * the_host ; 
    unsigned short the_current_logical_card_identifier ;
    SOCKET the_listening_socket ;
    SOCKET the_connection_socket;
    int the_verbosity ;
    TLP224 the_TLP224 ;
    bool use_GET_RESPONSE ;
    RAPDU_Packet the_saved_response_data ;
  } ;

#endif  // !defined(__TLP_APDU_CLIENT_H_INCLUDED__)
