// MemWdmDevice.cpp
// Implementation of MemWdmDevice device class
//
// Generated by DriverWizard version DriverStudio 3.1.0 (Build 1631)
// Requires Compuware's DriverWorks classes
//

#pragma warning(disable:4065) // Allow switch statement with no cases
		  
#include <vdw.h>
#include "..\MemWdmDeviceinterface.h"

#include "MemWdm.h"
#include "MemWdmDevice.h"
#include "..\MemWdmioctl.h"

#pragma hdrstop("MemWdm.pch")

extern KDebugOnlyTrace t;	// Global driver trace object	

//SIO Address 0x2E
#define SIO_2E		0x00	// offset 0 - 2E
#define SIO_2F		0x01	// offset 1 - 2F



GUID MemWdmDevice_Guid = MemWdmDevice_CLASS_GUID;
////////////////////////////////////////////////////////////////////////
//  MemWdmDevice::MemWdmDevice
//
//	Routine Description:
//		This is the constructor for the Functional Device Object, or FDO.
//		It is derived from KPnpDevice, which builds in automatic
//	    dispatching of subfunctions of IRP_MJ_POWER and IRP_MJ_PNP to
//		virtual member functions.
//
//	Parameters:
//		Pdo - Physical Device Object - this is a pointer to a system
//			device object that represents the physical device.
//
//		Unit - Unit number. This is a number to append to the device's
//			base device name to form the Logical Device Object's name
//
//	Return Value:
//		None   
//
//	Comments:
//		The object being constructed contains a data member (m_Lower) of type
//		KPnpLowerDevice. By initializing it, the driver binds the FDO to the
//		PDO and creates an interface to the upper edge of the system class driver.
//
MemWdmDevice::MemWdmDevice(PDEVICE_OBJECT Pdo, ULONG Unit) :
	KPnpDevice(Pdo, &MemWdmDevice_Guid)
{
	t << "Entering MemWdmDevice::MemWdmDevice (constructor)\n";


	// Check constructor status
    if ( ! NT_SUCCESS(m_ConstructorStatus) )
	{
	    return;
	}

	// Remember our unit number
	m_Unit = Unit;

	// Initialize the lower device
	m_Lower.Initialize(this, Pdo);

    // Inform the base class of the lower edge device object
	SetLowerDevice(&m_Lower);

	// Initialize the PnP Policy settings to the "standard" policy
	SetPnpPolicy();

	// Initialize the Power Policy settings to the "standard" policy
	SetPowerPolicy();

}


////////////////////////////////////////////////////////////////////////
//  MemWdmDevice::~MemWdmDevice
//
//	Routine Description:
//		This is the destructor for the Functional Device Object, or FDO.
//
//	Parameters:
//		None
//
//	Return Value:
//		None
//
//	Comments:
//		None
//

MemWdmDevice::~MemWdmDevice()
{
	t << "Entering MemWdmDevice::~MemWdmDevice() (destructor)\n";
}


////////////////////////////////////////////////////////////////////////
//  MemWdmDevice::DefaultPnp
//
//	Routine Description:
//		Default handler for IRP_MJ_PNP
//
//	Parameters:
//		I - Current IRP
//
//	Return Value:
//		NTSTATUS - Result returned from lower device
//
//	Comments:
//		This routine just passes the IRP through to the lower device. It is 
//		the default handler for IRP_MJ_PNP. IRPs that correspond to
//		any virtual members of KpnpDevice that handle minor functions of
//		IRP_MJ_PNP and that are not overridden get passed to this routine.
//

NTSTATUS MemWdmDevice::DefaultPnp(KIrp I) 
{
	t << "Entering MemWdmDevice::DefaultPnp\n" << I << EOL;

	I.ForceReuseOfCurrentStackLocationInCalldown();
	return m_Lower.PnpCall(this, I);
}

////////////////////////////////////////////////////////////////////////
//  MemWdmDevice::DefaultPower
//
//	Routine Description:
//		Default handler for IRP_MJ_POWER 
//  
//	Parameters:
//		I - Current IRP
//
//	Return Value:
//		NTSTATUS - Result returned from lower device
//
//	Comments:
//		This routine just passes the IRP through to the lower device. It is 
//		the default handler for IRP_MJ_POWER.
//

NTSTATUS MemWdmDevice::DefaultPower(KIrp I) 
{
	t << "Entering MemWdmDevice::DefaultPower\n" << I << EOL;

	I.IndicatePowerIrpProcessed();
	I.CopyParametersDown();
	return m_Lower.PnpPowerCall(this, I);
}

////////////////////////////////////////////////////////////////////////////////
//  MemWdmDevice::SystemControl
//
//	Routine Description:
//		Default handler for IRP_MJ_SYSTEM_CONTROL
//
//	Parameters:
//		I - Current IRP
//
//	Return Value:
//		NTSTATUS - Result returned from lower device
//
//	Comments:
//		This routine just passes the IRP through to the next device since this driver
//		is not a WMI provider.
//

NTSTATUS MemWdmDevice::SystemControl(KIrp I) 
{
	t << "Entering MemWdmDevice::SystemControl\n";

	I.ForceReuseOfCurrentStackLocationInCalldown();
	return m_Lower.PnpCall(this, I);
}

////////////////////////////////////////////////////////////////////////
//  MemWdmDevice::Invalidate
//
//	Routine Description:
//		Calls Invalidate methods for system resources
//
//	Parameters:
//		None
//
//	Return Value:
//		None
//
//	Comments:
//		This function is called from OnStopDevice, OnRemoveDevice and
//		OnStartDevice (in error conditions).  It calls the Invalidate
//		member funcitons for each resource to free the underlying system
//		resource if allocated.  It is safe to call Invalidate more than
//		once for a resource, or for an uninitialized resource.

VOID MemWdmDevice::Invalidate()
{

	// For each I/O port mapped region, release the underlying system resource.
	m_IoPortRangeGPIO.Invalidate();
	m_MemRangeEBDA.Invalidate();
}

////////////////////////////////////////////////////////////////////////
//  MemWdmDevice::OnStartDevice
//
//	Routine Description:
//		Handler for IRP_MJ_PNP subfcn IRP_MN_START_DEVICE
//
//	Parameters:
//		I - Current IRP
//
//	Return Value:
//		NTSTATUS - Result code
//
//	Comments:
//		Initialize the physical device. Typically, the driver initializes
//		physical resources here.  Call I.AllocatedResources() for a list
//		of the raw resources that the system has assigned to the device,
//		or I.TranslatedResources() for the translated resource list.
//

NTSTATUS MemWdmDevice::OnStartDevice(KIrp I)
{
	t << "Entering MemWdmDevice::OnStartDevice\n";

	//
	// initialize the IO range for access
	//
	// GPIO
	NTSTATUS status = m_IoPortRangeGPIO.Initialize (m_nPortBaseAddrGPIO, TRUE, 0x400);

	// Memory Base for EBDA
	if (NT_SUCCESS(status)) {
		status = m_MemRangeEBDA.Initialize (m_nMemBaseAddrEBDA, 0x100);	// 0x100
	}

	I.Information() = 0;

	// The default Pnp policy has already cleared the IRP with the lower device
	// Initialize the physical device object.

	// Get the list of raw resources from the IRP
	PCM_RESOURCE_LIST pResListRaw = I.AllocatedResources();
	// Get the list of translated resources from the IRP
	PCM_RESOURCE_LIST pResListTranslated = I.TranslatedResources();

	if (!NT_SUCCESS(status))
	{
		Invalidate();
		return status;		
	}

	return status;
}


////////////////////////////////////////////////////////////////////////
//  MemWdmDevice::OnStopDevice
//
//	Routine Description:
//		Handler for IRP_MJ_PNP subfcn IRP_MN_STOP_DEVICE
//
//	Parameters:
//		I - Current IRP
//
//	Return Value:
//		NTSTATUS - Result code
//
//	Comments:
//		The system calls this when the device is stopped.
//		The driver should release any hardware resources
//		in this routine.
//
//		The base class passes the irp to the lower device.
//

NTSTATUS MemWdmDevice::OnStopDevice(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;

	t << "Entering MemWdmDevice::OnStopDevice\n";

	// Device stopped, release the system resources.
	Invalidate();

// TODO:	Add device-specific code to stop your device   

	return status;
	
	// The following macro simply allows compilation at Warning Level 4
	// If you reference this parameter in the function simply remove the macro.
	UNREFERENCED_PARAMETER(I);
}

////////////////////////////////////////////////////////////////////////
//  MemWdmDevice::OnRemoveDevice
//
//	Routine Description:
//		Handler for IRP_MJ_PNP subfcn IRP_MN_REMOVE_DEVICE
//
//	Parameters:
//		I - Current IRP
//
//	Return Value:
//		NTSTATUS - Result code
//
//	Comments:
//		The system calls this when the device is removed.
//		Our PnP policy will take care of 
//			(1) giving the IRP to the lower device
//			(2) detaching the PDO
//			(3) deleting the device object
//

NTSTATUS MemWdmDevice::OnRemoveDevice(KIrp I)
{
	t << "Entering MemWdmDevice::OnRemoveDevice\n";

	// Device removed, release the system resources.
	Invalidate();

// TODO:	Add device-specific code to remove your device   

	return STATUS_SUCCESS;

	// The following macro simply allows compilation at Warning Level 4
	// If you reference this parameter in the function simply remove the macro.
	UNREFERENCED_PARAMETER(I);
}

////////////////////////////////////////////////////////////////////////
//  MemWdmDevice::OnDevicePowerUp
//
//	Routine Description:
//		Handler for IRP_MJ_POWER with minor function IRP_MN_SET_POWER
//		for a request to go to power on state from low power state
//
//	Parameters:
//		I - IRP containing POWER request
//
//	Return Value:
//		NTSTATUS - Status code indicating success or failure
//
//	Comments:
//		This routine implements the OnDevicePowerUp function.
//		This function was called by the framework from the completion
//		routine of the IRP_MJ_POWER dispatch handler in KPnpDevice.
//		The bus driver has completed the IRP and this driver can now
//		access the hardware device.  
//		This routine runs at dispatch level.
//	

NTSTATUS MemWdmDevice::OnDevicePowerUp(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;

	t << "Entering MemWdmDevice::OnDevicePowerUp\n";

// TODO:	Service the device.
//			Restore any context to the hardware device that
//			was saved during the handling of a power down request.
//			See the OnDeviceSleep function.
//			Do NOT complete this IRP.
//
	return status;

	// The following macro simply allows compilation at Warning Level 4
	// If you reference this parameter in the function simply remove the macro.
	UNREFERENCED_PARAMETER(I);
}

////////////////////////////////////////////////////////////////////////
//  MemWdmDevice::OnDeviceSleep
//
//	Routine Description:
//		Handler for IRP_MJ_POWER with minor function IRP_MN_SET_POWER
//		for a request to go to a low power state from a high power state
//
//	Parameters:
//		I - IRP containing POWER request
//
//	Return Value:
//		NTSTATUS - Status code indicating success or failure
//
//	Comments:
//		This routine implements the OnDeviceSleep function.
//		This function was called by the framework from the IRP_MJ_POWER 
//		dispatch handler in KPnpDevice prior to forwarding to the PDO.
//		The hardware has yet to be powered down and this driver can now
//		access the hardware device.  
//		This routine runs at passive level.
//	

NTSTATUS MemWdmDevice::OnDeviceSleep(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;

	t << "Entering MemWdmDevice::OnDeviceSleep\n";

// TODO:	Service the device.
//			Save any context to the hardware device that will be required 
//			during a power up request. See the OnDevicePowerUp function.
//			Do NOT complete this IRP.  The base class handles forwarding
//			this IRP to the PDO.
//
	return status;

	// The following macro simply allows compilation at Warning Level 4
	// If you reference this parameter in the function simply remove the macro.
	UNREFERENCED_PARAMETER(I);
}

////////////////////////////////////////////////////////////////////////
//  MemWdmDevice::Create
//
//	Routine Description:
//		Handler for IRP_MJ_CREATE
//
//	Parameters:
//		I - Current IRP
//
//	Return Value:
//		NTSTATUS - Result code
//
//	Comments:
//

NTSTATUS MemWdmDevice::Create(KIrp I)
{
	NTSTATUS status;

	t << "Entering MemWdmDevice::Create, " << I << EOL;

// TODO: Add driver specific create handling code here

	// Generally a create IRP is targeted at our FDO, so we don't need
	// to pass it down to the PDO.  We have found for some devices, the
	// PDO is not expecting this Irp and returns an error code.
	// The default wizard code, therefore completes the Irp here using
	// PnpComplete().  The following commented code could be used instead
	// of PnpComplete() to pass the Irp to the PDO, which would complete it.
	//
//	I.ForceReuseOfCurrentStackLocationInCalldown();
//	status = m_Lower.PnpCall(this, I);

	status = I.PnpComplete(this, STATUS_SUCCESS, IO_NO_INCREMENT);

	t << "MemWdmDevice::Create Status " << (ULONG)status << EOL;

	return status;
}

////////////////////////////////////////////////////////////////////////
//  MemWdmDevice::Close
//
//	Routine Description:
//		Handler for IRP_MJ_CLOSE
//
//	Parameters:
//		I - Current IRP
//
//	Return Value:
//		NTSTATUS - Result code
//
//	Comments:
//

NTSTATUS MemWdmDevice::Close(KIrp I)
{
	NTSTATUS status;

	t << "Entering MemWdmDevice::Close, " << I << EOL;

// TODO: Add driver specific close handling code here

	// Generally a close IRP is targeted at our FDO, so we don't need
	// to pass it down to the PDO.  We have found for some devices, the
	// PDO is not expecting this Irp and returns an error code.
	// The default wizard code, therefore completes the Irp here using
	// PnpComplete().  The following commented code could be used instead
	// of PnpComplete() to pass the Irp to the PDO, which would complete it.
	//
//	I.ForceReuseOfCurrentStackLocationInCalldown();
//	status = m_Lower.PnpCall(this, I);

	status = I.PnpComplete(this, STATUS_SUCCESS, IO_NO_INCREMENT);

	t << "MemWdmDevice::Close Status " << (ULONG)status << EOL;

    return status;
}

////////////////////////////////////////////////////////////////////////
//  MemWdmDevice::Cleanup
//
//	Routine Description:
//		Handler for IRP_MJ_CLEANUP	
//
//	Parameters:
//		I - Current IRP
//
//	Return Value:
//		NTSTATUS - Result code
//
//	Comments:
//

NTSTATUS MemWdmDevice::CleanUp(KIrp I)
{
	t << "Entering CleanUp, " << I << EOL;

// TODO:	Insert your code to respond to the CLEANUP message.
//			This code cleans up the single Wizard created queue.  If you
//			have created additional queues,	or have any outstanding Irps
//			stored in some other fashion in your driver, you should clean
//			these up as well for the file object specified in the cleanup Irp.

	m_DriverManagedQueue.PnpCleanUp(this, I.FileObject());
	return I.PnpComplete(this, STATUS_SUCCESS);
}

////////////////////////////////////////////////////////////////////////
//  MemWdmDevice::DeviceControl
//
//	Routine Description:
//		Handler for IRP_MJ_DEVICE_CONTROL
//
//	Parameters:
//		I - Current IRP
// 
//	Return Value:
//		None
//
//	Comments:
//		This routine is the first handler for Device Control requests.
//		Some function codes may be handled immediately, 
//		while others may be serialized through the StartIo routine.
//		
//		The KPnpDevice class handles restricting IRP flow
//		if the device is stopping or being removed.
//

NTSTATUS MemWdmDevice::DeviceControl(KIrp I) 
{
	NTSTATUS status;

	t << "Entering MemWdmDevice::Device Control, " << I << EOL;
	switch (I.IoctlCode())
	{
		case MEMWDM_IOCTL_GPIO:
			status = MEMWDM_IOCTL_GPIO_Handler(I);
			break;

		case MEMWDM_IOCTL_EBDA:
			status = MEMWDM_IOCTL_EBDA_Handler(I);
			break;

		default:
			// Unrecognized IOCTL request
			status = STATUS_INVALID_PARAMETER;
			break;
	}

	// If the IRP was queued, or its IOCTL handler deferred processing using some
	// driver specific scheme, the status variable is set to STATUS_PENDING.
	// In this case we simply return that status, and the IRP will be completed
	// later.  Otherwise, complete the IRP using the status returned by the
	// IOCTL handler.
	if (status == STATUS_PENDING)
	{
		return status;
	}
	else
	{
		return I.PnpComplete(this, status);
	}
}

////////////////////////////////////////////////////////////////////////
//  MemWdmDevice::MEMWDM_IOCTL_GPIO_Handler
//
//	Routine Description:
//		Handler for IO Control Code MEMWDM_IOCTL_GPIO
//
//	Parameters:
//		I - IRP containing IOCTL request
//
//	Return Value:
//		NTSTATUS - Status code indicating success or failure
//
//	Comments:
//		This routine implements the MEMWDM_IOCTL_GPIO function.
//		This routine runs at passive level.
//

NTSTATUS MemWdmDevice::MEMWDM_IOCTL_GPIO_Handler(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;
	USHORT GPIO_Addr;

	t << "Entering MemWdmDevice::MEMWDM_IOCTL_GPIO_Handler, " << I << EOL;

	PVOID ioBuffer  = I.IoctlBuffer();
	ULONG ioInSize  = I.IoctlInputBufferSize();
	ULONG ioOutSize = I.IoctlOutputBufferSize();

	if (ioInSize == 4) { //Read

		GPIO_Addr = (*((UCHAR*)ioBuffer + 1)) * 256 + (*(UCHAR*)ioBuffer);
		*(UCHAR*)ioBuffer = (UCHAR)m_IoPortRangeGPIO.inb (GPIO_Addr);

		I.Information() = sizeof(UCHAR);
	}
	else if (ioInSize == 5) { //Write

		GPIO_Addr = (*((UCHAR*)ioBuffer + 1)) * 256 + (*(UCHAR*)ioBuffer);
		m_IoPortRangeGPIO.outb (GPIO_Addr, *((UCHAR*)ioBuffer + 4));

		I.Information() = 0;
	}
	else {
		I.Information() = 0;
	}

	return status;
}

////////////////////////////////////////////////////////////////////////
//  MemWdmDevice::MEMWDM_IOCTL_EBDA_Handler
//
//	Routine Description:
//		Handler for IO Control Code MEMWDM_IOCTL_EBDA
//
//	Parameters:
//		I - IRP containing IOCTL request
//
//	Return Value:
//		NTSTATUS - Status code indicating success or failure
//
//	Comments:
//		This routine implements the MEMWDM_IOCTL_EBDA function.
//		This routine runs at passive level.
//

NTSTATUS MemWdmDevice::MEMWDM_IOCTL_EBDA_Handler(KIrp I)
{
	NTSTATUS status = STATUS_SUCCESS;
	USHORT Mem_Offset;
	USHORT RWCount, i;

	t << "Entering MemWdmDevice::MEMWDM_IOCTL_EBDA_Handler, " << I << EOL;

	PVOID ioBuffer  = I.IoctlBuffer();
	ULONG ioInSize  = I.IoctlInputBufferSize();
	ULONG ioOutSize = I.IoctlOutputBufferSize();

//	Mem_Offset = *(UCHAR*)ioBuffer
//			+ (*((UCHAR*)ioBuffer + 1)) * 256
	Mem_Offset = *(USHORT*)ioBuffer;

//	RWCount = *((UCHAR*)ioBuffer + 3)
//			+ (*((UCHAR*)ioBuffer + 4)) * 256;		// Maxmum 65536 byte
	RWCount = *((USHORT*)ioBuffer + 1);

	t << "MemWdmDevice::EBDA Base 9FC0:0     " << Mem_Offset << EOL;
	t << "MemWdmDevice::EBDA R/W Bytes Count " << RWCount << EOL;


	if (ioOutSize == 0x00) {
		// Write Acess
		for (i=0; i<RWCount; i++) {
			m_MemRangeEBDA.outb (Mem_Offset + i, *((UCHAR*)ioBuffer + 4 + i));
		}
		I.Information() = 0;
	}
	else {
		// Read Access
		if (ioOutSize != RWCount) {
			I.Information() = 0;
			return STATUS_INVALID_PARAMETER;
		}
		for (i=0; i<RWCount; i++) {
			*((UCHAR*)ioBuffer + i) = (UCHAR)m_MemRangeEBDA.inb (Mem_Offset + i);
			t << "MemWdmDevice::BDA ReadData " << *((UCHAR*)ioBuffer + i) << EOL;
		}
		I.Information() = RWCount;
	}
	
	return status;
}

///////////////////////////////////////////////////////////////////////////////
//  MemWdmDevice_DriverManagedQueue::StartIo
//	
//	Routine Description:
//		This routine is called when an IRP is taken off
//		the Driver Managed Queue (used for serializing I/O) and
//		presented for processing.
//
//	Parameters:
//		I - IRP removed from queue
//
//	Return Value:
//		None
//
//	Comments:
//

VOID MemWdmDevice_DriverManagedQueue::StartIo(KIrp I)
{
	t  << "Entering MemWdmDevice_DriverManagedQueue StartIo, " << I;

	// The KDriverManagedQueueEx class gives us the Irp in a non-cancelable state
	// (cancel routine set to NULL) so we can process it without having to worry
	// about clearing the cancel routine first, as is the case with system queuing,
	// or the legacy class KDriverManagedQueue. You may want to set a different cancel
	// routine here, or at other points during the processing of this Irp.

	// Find the device class so we can call the serialized
	// routines in the device class.  The handlers can be
	// moved to the DriverManagedQueue class if it is more
	// convenient.
	MemWdmDevice *pDev = (MemWdmDevice *) KDevicePTR(I.DeviceObject());

	// Start processing request.

	// Switch on the IRP's function:
	switch (I.MajorFunction())
	{
		case IRP_MJ_DEVICE_CONTROL:
			switch (I.IoctlCode())
			{
				default:
					// We queued a request that shouldn't have been queued
					// (should never get here)
					ASSERT(FALSE);
					break;
			}
			break;

		default:
			// Error - unexpected IRP received
			// NextIrp completes this IRP and starts processing
			// for the next IRP in the queue.
			ASSERT(FALSE);
			I.Status() = STATUS_INVALID_PARAMETER;
			PnpNextIrp(I);
			break;
	}
}

////////////////////////////////////////////////////////////////////////
//  MemWdmDevice::SetParams
//
//	Routine Description:
//		set parameters for initializing/accessing the IO registers
//
//	Parameters:
//		portBaseAddr	-	base IO address
//		unlockID		-	address to unlock register
//		brightDownKey	-	address of register for down key
//		brightUpKey		-	address of register for up key
//      forceRechareKey -   address of register for force recharge key
//
//	Return Value:
//		None
//
//	Comments:
//		None
//

void MemWdmDevice::SetParams (USHORT portBaseAddr0, ULONG memBaseAddr0)
{
	t << "Entering MemWdmDevice::SetParams()\n";

	m_nPortBaseAddrGPIO = portBaseAddr0;
	m_nMemBaseAddrEBDA = memBaseAddr0;
}

void MemWdmDevice::WaitLoop (USHORT wcount)
{

	USHORT count;

	count = 0;									// Wait
	do {
		count++;
	} while (count < wcount);

	return;
}
