// PIV VSCR Installer.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "io.h"
#include "Winsvc.h"

#define PIVVSCR_SERVICE_NAME			_T("PIV Virtual Smart Card Reader")

void PrintHelp()
{
	_tprintf(_T("PIV VSCR Installer Version 1.0\n\n"));
	_tprintf(_T("usage: pivvscr_inst [-option] [file path to service executable] [file path to service DLL]\n\n"));
	_tprintf(_T("   -i  Install PIV Virtual Smart Card Reader service\n"));
	_tprintf(_T("   -r  Remove PIV Virtual Smart Card Reader service\n\n"));
}

int _tmain(int argc, _TCHAR* argv[])
{
	int dRetVal = -1;

	// make sure a command line option is specified
	if ((argc != 2) && (argc != 4)) {
		PrintHelp();
		return -1;
	}

	// display help screen if needed
	if (_tcscmp(argv[1], _T("-?")) == 0) {

		PrintHelp();
		return -1;
	}

	// install PIV Virtual Smart Card Reader service
	if (_tcscmp(argv[1], _T("-i")) == 0) {

		// make sure service executable and dll exist
		if (_taccess(argv[2], 0) == 0 && _taccess(argv[3], 0) == 0) {
			TCHAR tcSystemRoot[MAX_PATH], tcExeName[_MAX_FNAME], tcDllName[_MAX_FNAME],
				  tcExeExt[_MAX_EXT], tcDllExt[_MAX_EXT], 
				  tcExeDestination[MAX_PATH], tcDllDestination[MAX_PATH];

			// get value of "SystemRoot" environment variable
			GetEnvironmentVariable(_T("SystemRoot"), tcSystemRoot, MAX_PATH);

			// if necessary, append trailing backslash to system root path
			if (tcSystemRoot[_tcslen(tcSystemRoot)-1] != _T('\\'))
				_tcscat(tcSystemRoot, _T("\\"));

			_tsplitpath(argv[2], NULL, NULL, tcExeName, tcExeExt);
			_tsplitpath(argv[3], NULL, NULL, tcDllName, tcDllExt);
			_stprintf(tcExeDestination, _T("%ssystem32\\%s%s"), tcSystemRoot, tcExeName, tcExeExt);
			_stprintf(tcDllDestination, _T("%ssystem32\\%s%s"), tcSystemRoot, tcDllName, tcDllExt);

			// copy the service executable file to the system directory
			if (CopyFile(argv[2], tcExeDestination, FALSE) && CopyFile(argv[3], tcDllDestination, FALSE)) {
				SC_HANDLE hSCManager;

				// open service control manager
				hSCManager = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);

				if (hSCManager != NULL) {
					SC_HANDLE hService;
					TCHAR tcBinaryPathName[MAX_PATH];

					_stprintf(tcBinaryPathName, _T("%%SystemRoot%%\\system32\\%s%s"), 
						tcExeName, tcExeExt);

					// create our service
					hService = CreateService( 
									hSCManager,						// SCManager database 
									PIVVSCR_SERVICE_NAME,			// name of service 
									PIVVSCR_SERVICE_NAME,			// service name to display 
									SERVICE_ALL_ACCESS,				// desired access 
									SERVICE_WIN32_SHARE_PROCESS,	// service type 
									SERVICE_DEMAND_START,			// start type 
									SERVICE_ERROR_NORMAL,			// error control type 
									tcBinaryPathName,				// service's binary 
									NULL,							// no load ordering group 
									NULL,							// no tag identifier 
									NULL,							// no dependencies 
									NULL,							// LocalSystem account 
									NULL);							// no password 
			 
					if (hService != NULL) {
						HKEY hKey;
						TCHAR tcParametersKey[256];
						TCHAR tcDefaultHost[] = _T("127.0.0.1");
						DWORD dwPort = 9025;
						DWORD dwPollInterval = 100;
						DWORD dwAutoInsert = 1;

						_stprintf(tcParametersKey, _T("SYSTEM\\CurrentControlSet\\Services\\%s\\Parameters"),
							PIVVSCR_SERVICE_NAME);

						// create a "Parameters" subkey to hold host and port
						// info
						if (RegCreateKeyEx(
									HKEY_LOCAL_MACHINE,
									tcParametersKey,
									0,
									NULL,
									REG_OPTION_NON_VOLATILE,
									KEY_ALL_ACCESS,
									NULL,
									&hKey,
									NULL) == ERROR_SUCCESS) {

							// set default host information
							RegSetValueEx(
									hKey,
									_T("Host"),
									0,
									REG_SZ,
									(BYTE*) tcDefaultHost,
									(DWORD)(_tcslen(tcDefaultHost)+1) * sizeof(TCHAR));

							// set default port information
							RegSetValueEx(
									hKey,
									_T("Port"),
									0,
									REG_DWORD,
									(BYTE*) &dwPort,
									sizeof(dwPort));

							// set default poll interval information
							RegSetValueEx(
									hKey,
									_T("Poll Interval"),
									0,
									REG_DWORD,
									(BYTE*) &dwPollInterval,
									sizeof(dwPort));

							// set default auto insert value
							RegSetValueEx(
									hKey,
									_T("Auto Insert"),
									0,
									REG_DWORD,
									(BYTE*) &dwAutoInsert,
									sizeof(dwAutoInsert));

							RegCloseKey(hKey);
						}

						CloseServiceHandle(hService);
						dRetVal = 0;
					}
					else
						_tprintf(_T("Error: failed to create service\n"));

					CloseServiceHandle(hSCManager);
				}
				else
					_tprintf(_T("Error: could not open service control manager\n"));
			}
			else
				_tprintf(_T("Error: could not copy service executable and dll to system directory\n"));
		}
		else
		{
			if (_taccess(argv[2], 0) != 0)
				_tprintf(_T("Error: could not locate \"%s\"\n"), argv[2]);
			else
				_tprintf(_T("Error: could not locate \"%s\"\n"), argv[3]);
		}
	}
	// delete PIV Virtual Smart Card Reader service
	else if (_tcscmp(argv[1], _T("-r")) == 0) {
		SC_HANDLE hSCManager;

		// open service control manager
		hSCManager = OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);

		if (hSCManager != NULL) {
			SC_HANDLE hService;

			// open PIV Virtual Smart Card Reader service
			hService = OpenService(hSCManager, PIVVSCR_SERVICE_NAME, 
				SERVICE_ALL_ACCESS);

			if (hService != NULL) {
				SERVICE_STATUS ServiceStatus;

				// send a stop code to the service
				ControlService(hService, SERVICE_CONTROL_STOP, &ServiceStatus);

				// mark service for deletion
				DeleteService(hService);

				CloseServiceHandle(hService);
			}
			else
				_tprintf(_T("Error: could not locate PIV Virtual Smart Card Reader service\n"));

			CloseServiceHandle(hSCManager);

			TCHAR tcSystemRoot[MAX_PATH], tcInstalledExecutableFilePath[MAX_PATH];
			TCHAR *tcExeName = _T("pivvscr");
			TCHAR *tcExeExt = _T(".exe");

			// get value of "SystemRoot" environment variable
			GetEnvironmentVariable(_T("SystemRoot"), tcSystemRoot, MAX_PATH);

			// if necessary, append trailing backslash to system root path
			if (tcSystemRoot[_tcslen(tcSystemRoot)-1] != _T('\\'))
				_tcscat(tcSystemRoot, _T("\\"));

			_stprintf(tcInstalledExecutableFilePath, _T("%ssystem32\\%s%s"), tcSystemRoot, tcExeName, tcExeExt);

			// delete the service executable file in the system directory
			if (! DeleteFile(tcInstalledExecutableFilePath))
				_tprintf(_T("Error: could not delete \"%s\"\n"), tcInstalledExecutableFilePath);
		}
		else
			_tprintf(_T("Error: could not open service control manager\n"));

		dRetVal = 0;
	}
	else
		PrintHelp();

	return dRetVal;
}

