#include "stdafx.h"
#include "Debug.h"

static HANDLE g_hDebugFile = INVALID_HANDLE_VALUE;


// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Function:  SvcDebugHexToStr
//
void SvcDebugHexToStr(UCHAR* pData, ULONG ulDataSize, char* pszStr, ULONG ulStrSize)
{
	char* pChar = pszStr;
	ULONG ulBytesToConvert, x;
	BOOLEAN bMoreData = FALSE;

	if ((pszStr == NULL) || (ulStrSize == 0))
		return;

	if ((pData == NULL) || (ulDataSize == 0)) {

		pszStr[0] = '\0';
		return;
	}

	// Get number of bytes to convert. Each byte will be represented by 3
	// chars -- 2 digits and a trailing space.
	ulBytesToConvert = (ulStrSize - 1) / 3;

	// if output buffer is too small to hold converted data...
	if (ulBytesToConvert < ulDataSize) {

		// data must be at least two bytes
		if (ulDataSize <= 2) {

			pszStr[0] = '\0';
			return;
		}

		ulBytesToConvert -= 2;			// reserve 2 bytes to hold ".."
		bMoreData = TRUE;
	}
	else
		ulBytesToConvert = ulDataSize;

	// convert bytes to hex
	for (x=0; x < ulBytesToConvert; x++) {

		sprintf(pChar, "%02x ", pData[x]);
		pChar += 3;
	}

	// if not all the data was converted, add a ".." to the end of the string
	if (bMoreData)
		strcat(pszStr, "..");
}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Function:  SvcDebugOut
//
void SvcDebugOut(LPCTSTR lpszString)
{
    TCHAR tcBuffer[1024];

    if (_tcslen(lpszString) < 1000) {

        _stprintf(tcBuffer, _T("[PIV_VSCR] %s\n"), lpszString);
        OutputDebugStringA(tcBuffer);
    }
}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Function:  SvcDebugOut
//
void SvcDebugOut(LPCTSTR lpszString, DWORD dwStatus)
{ 
    TCHAR tcBuffer[1024];

    if (_tcslen(lpszString) < 1000) {

        _stprintf(tcBuffer, _T("[PIV_VSCR] %s%d\n"), lpszString, dwStatus);
        OutputDebugStringA(tcBuffer);
    }
}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Function:  SvcDebugCloseLog
//
void SvcDebugCloseLog()
{
	if (g_hDebugFile != INVALID_HANDLE_VALUE) {

        CloseHandle(g_hDebugFile);
		g_hDebugFile = INVALID_HANDLE_VALUE;
    }
}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Function:  SvcDebugPrintLog
//
void SvcDebugPrintLog(const char *message)
{
	DWORD bytesWritten;
	DWORD dwPos;

	// If the log file handle is invalid...
	if (g_hDebugFile == INVALID_HANDLE_VALUE)
	{
		// Try to get a valid file handle.
		g_hDebugFile = CreateFile(
							PIVVSCR_DEBUG_FILE,
							GENERIC_WRITE,
							FILE_SHARE_READ,
							NULL,
							OPEN_ALWAYS,
							FILE_ATTRIBUTE_NORMAL,
							NULL);
	}

	// If the log file handle is still invalid, error
	if (g_hDebugFile == INVALID_HANDLE_VALUE)
		return;

	// Go to the end of the file
	dwPos = SetFilePointer(g_hDebugFile, 0, NULL, FILE_END);

	// Lock at the end of file, on length of message +1
	if (!LockFile(g_hDebugFile, dwPos, 0, (DWORD) strlen(message), 0))
		return;

	// Write the message to the file
	if (!WriteFile(g_hDebugFile, message, (DWORD) strlen(message), &bytesWritten, NULL)) {

		UnlockFile(g_hDebugFile, dwPos, 0, (DWORD) strlen(message), 0);
		return;
	}

	// Unlock the file
	UnlockFile(g_hDebugFile, dwPos, 0, (DWORD) strlen(message),0);
}

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Function:  SvcDebugPrintLog2
//
void SvcDebugPrintLog2(const char *message, int len)
{
	DWORD bytesWritten;
	DWORD dwPos;

	// If the log file handle is invalid...
	if (g_hDebugFile == INVALID_HANDLE_VALUE)
	{
		// Try to get a valid file handle
		g_hDebugFile = CreateFile(
							PIVVSCR_DEBUG_FILE,
							GENERIC_WRITE,
							FILE_SHARE_READ,
							NULL,
							OPEN_ALWAYS,
							FILE_ATTRIBUTE_NORMAL,
							NULL);
	}
	// If the log file handle is still invalid, error
	if (g_hDebugFile == INVALID_HANDLE_VALUE)
		return;

	// Go to the end of the file
	dwPos = SetFilePointer(g_hDebugFile, 0, NULL, FILE_END);

	// Lock at the end of file, on length of message +1
	if (!LockFile(g_hDebugFile, dwPos, 0, (DWORD) strlen(message), 0))
		return;

	// Write the message to the file
	if (!WriteFile(g_hDebugFile, message, len, &bytesWritten, NULL)) {

		UnlockFile(g_hDebugFile, dwPos, 0, (DWORD) strlen(message), 0);
		return;
	}

	// Unlock the file
	UnlockFile(g_hDebugFile, dwPos, 0, (DWORD) strlen(message),0);
}