#pragma once

#include "EventLog.h"

class CVirtualSmartCardReader
{
public:
	enum PIV_EVENT_TYPE {PIV_EVENT_SUCCESS, PIV_EVENT_INFO, PIV_EVENT_WARNING, PIV_EVENT_ERROR};

//	Constructors
	CVirtualSmartCardReader(void);
	~CVirtualSmartCardReader(void);

//	Implementation
	BOOL isConnected();

	BOOL Initialize();
    void Process();

    BOOL Connect();
    void Disconnect();
    void Terminate();
	void InsertCard();
	void ForceCardRemoval();
	BOOL AutoInsertEnabled() { return m_bAutoInsert; }

private:
	void ReportPIVEvent(enum PIV_EVENT_TYPE eType, const char* format, ...)
	{
        static char Buffer[4096 + 1] = {0};
        va_list vals;
        va_start ( vals , format ) ;     /* Initialize variable arguments. */
        _vsnprintf ( Buffer , 4096 , format , vals ) ;
        
		switch (eType) {
		case PIV_EVENT_SUCCESS:
			::ReportDefaultSuccessEvent(g_AppName, Buffer);
			break;

		case PIV_EVENT_INFO:
			::ReportDefaultInformationEvent(g_AppName, Buffer);
			break;

		case PIV_EVENT_WARNING:
			::ReportDefaultWarningEvent(g_AppName, Buffer);
			break;

		case PIV_EVENT_ERROR:
			::ReportDefaultErrorEvent(g_AppName, Buffer);
			break;
		}
	}

    void error_message(const char * format, ...)
    {
        static char Buffer[4096 + 1] = {0};
        va_list vals;
        va_start ( vals , format ) ;     /* Initialize variable arguments. */
        _vsnprintf ( Buffer , 4096 , format , vals ) ;
        //MessageBox ( NULL , Buffer , "Virtual Smart Card Reader Interface" , MB_OK ) ;
		::ReportDefaultErrorEvent(g_AppName, Buffer);
    }

	BOOL GetConfigurationFromRegistry();
	ULONG ReadDW_EBDA(USHORT, UCHAR);
	BOOL  WriteDW_EBDA(USHORT, ULONG, UCHAR);

	HANDLE m_hDevice;					// the handle to the device
    CString m_hostname;
    DWORD m_port;
	BOOL m_bAutoInsert;
	int m_iPacketCnt;
	BOOL m_bWDMFlag;
	BOOL m_bConnectedtoRefImp;
	UCHAR m_byDataIn[8];
	UCHAR m_byDataOut[4];
};
