#include "stdafx.h"
#include "Debug.h"
#include "servicethread.h"

CServiceThread::CServiceThread(DWORD dwNewSleepMilliseconds)
{
	m_dwSleepMilliseconds = dwNewSleepMilliseconds;
	m_hThread = NULL;
}

CServiceThread::~CServiceThread(void)
{
	// if the thread is running then stop it
	Stop();
}

DWORD CServiceThread::Start()
{
	DWORD dwStatus = ERROR_FUNCTION_FAILED;

	// perform initialization routines
	if (m_VirtualSmartCardReader.Initialize()) {

		// create thread to process messages
		m_hThread = CreateThread(
							NULL,
							0, 
							(LPTHREAD_START_ROUTINE)RunLoop, 
							(LPVOID)this, 
							0, 
							NULL);

		if (m_hThread == NULL)
			dwStatus = ERROR_INVALID_HANDLE;
		else
			dwStatus = ERROR_SUCCESS;
	}

	return dwStatus;
}

DWORD CServiceThread::Start(DWORD argc, LPTSTR *argv, DWORD *specificError)
{
//	ThreadStart* threadStart = new ThreadStart(this, runLoop);
//	thread = new Thread(threadStart);
//	thread->Start();
	DWORD dwRetCode = Start();

	if (dwRetCode != ERROR_SUCCESS)
		*specificError = GetLastError();

	return dwRetCode;
}

DWORD CServiceThread::Stop(void)
{
	DWORD dwStatus = ERROR_SUCCESS;

	// stop our thread
	if (m_hThread != NULL) {

		// if a card is currently inserted, we want to remove it before 
		// stopping the service
		m_VirtualSmartCardReader.ForceCardRemoval();

		// set global flag indicating the thread should be stopped
		m_bStopping = true;

		// Clearing m_bStopping will cause the thread to stop.  If it hasn't
		// stopped then give it a second before we terminate the thread.
		if (WaitForSingleObject(m_hThread, 1000) == WAIT_TIMEOUT)
			TerminateThread(m_hThread, -1);

		// terminate the Virtual Smart Card Reader
		m_VirtualSmartCardReader.Terminate();

		CloseHandle(m_hThread);
		m_hThread = NULL;
	}

	SvcDebugCloseLog();

	return dwStatus;
}

void CServiceThread::CheckAutoInsert()
{
	// if auto insert set, insert card into reader
	if (m_VirtualSmartCardReader.AutoInsertEnabled())
		m_VirtualSmartCardReader.InsertCard();
}

static DWORD __stdcall RunLoop(CServiceThread *thread)
{
	thread->m_bStopping = false;

	while (!thread->m_bStopping)
	{
		// do work
		thread->m_VirtualSmartCardReader.Process();
		Sleep(thread->GetSleepMilliseconds());
	}

	return 0;
}